@@ -12,6 +12,7 @@ import astropy.units as asunits
 
 
 from . import _generic_check
+from . import _generic_utils
 
 
 _DRESERVED_KEYS = {
@@ -146,7 +147,14 @@ def _check_conflicts(dd=None, dd0=None, dd_name=None, returnas=None):
                     or (
                         not isinstance(v0[kk], np.ndarray)
                         and not scpsp.issparse(v0[kk])
-                        and v0[kk] == dd0[k0][kk]
+                        and (
+                            v0[kk] == dd0[k0][kk]
+                            or (
+                                # asunit.Unit vs str should be the same
+                                kk == 'units'
+                                and str(v0[kk]) == str(dd0[k0][kk])
+                            )
+                        )
                     )
                 )
             )
@@ -198,12 +206,18 @@ def _check_conflicts(dd=None, dd0=None, dd_name=None, returnas=None):
 
 
 def _check_remove(key=None, dkey=None, name=None):
-    c0 = isinstance(key, str) and key in dkey.keys()
-    c1 = (
-        isinstance(key, list)
-        and all([isinstance(kk, str) and kk in dkey.keys() for kk in key])
+
+    # -----------------------
+    # check basic conformity
+
+    if isinstance(key, str):
+        key = [key]
+
+    c0 = (
+        isinstance(key, (list, tuple, set))
+        and all([isinstance(kk, str) for kk in key])
     )
-    if not (c0 or c1):
+    if not c0:
         msg = (
             """
             Removed item must be a str already in self.d{}
@@ -213,8 +227,12 @@ def _check_remove(key=None, dkey=None, name=None):
             """.format(name, key, sorted(dkey.keys()))
         )
         raise Exception(msg)
-    if c0:
-        key = [key]
+
+    # -----------------------
+    # only keep relevant keys
+
+    key = [k0 for k0 in key if k0 in dkey.keys()]
+
     return key
 
 
@@ -327,7 +345,8 @@ def _remove_ref(
     for k0 in key:
         # Remove orphan ddata
         for k1 in dref0[k0]['ldata']:
-            del ddata0[k1]
+            if k1 in ddata0.keys():
+                del ddata0[k1]
         del dref0[k0]
 
     # Double-check consistency
@@ -474,7 +493,7 @@ def _remove_obj(
             del dobj0[which][kk]
 
     # cleanup
-    if propagate is True and len(dobj0.get(which, {})) == 0:
+    if propagate is True and dobj0.get(which) is not None and len(dobj0[which]) == 0:
         del dobj0[which]
 
     return _consistency(
@@ -590,14 +609,14 @@ def _check_dref(
             isinstance(v0, dict)
             and (
                 isinstance(v0.get('data'), (np.ndarray, list, tuple))
-                or isinstance(v0.get('size'), (int, np.int_))
+                or isinstance(v0.get('size'), (int, np.integer))
             )
         )
         if not c0:
             msg = "v0 must be a dict with either 'data' or 'size'"
             raise Exception(msg)
 
-        if isinstance(v0.get('size'), (int, np.int_)):
+        if isinstance(v0.get('size'), (int, np.integer)):
             dref[k0]['size'] = int(v0['size'])
 
         dref2[key] = dict(dref[k0])
@@ -892,7 +911,7 @@ def _check_ddata(
                                 v0.get('ref') is None
                                 or isinstance(v0.get('ref'), str)
                                 or (
-                                    isinstance(v0.get('ref'), tuple)
+                                    isinstance(v0.get('ref'), (list, tuple))
                                     and all([
                                         isinstance(rr, str)
                                         for rr in v0['ref']
@@ -965,6 +984,8 @@ def _check_ddata(
         # find ref
         if isinstance(v0['ref'], str):
             ddata[k0]['ref'] = (v0['ref'],)
+        elif isinstance(v0['ref'], list):
+            ddata[k0]['ref'] = tuple(v0['ref'])
 
         _check_data_ref(
             k0=k0,
@@ -1020,13 +1041,15 @@ def _check_dobj(
 
     # ----------------
     # Trivial case
+    # ----------------
+
     if dobj in [None, {}]:
         return {}
 
-    # ----------------
-    # Check conformity
+    # ------------
+    # Check type
+    # ------------
 
-    # map possible non-conformities
     if not isinstance(dobj, dict):
         msg = (
             "Arg dobj must be a dict!\n"
@@ -1034,48 +1057,69 @@ def _check_dobj(
         )
         raise Exception(msg)
 
+    # -----------------------------
     # Map possible non-conformities
+    # -----------------------------
+
     dc = {}
     dobj2 = {}
     for k0, v0 in dobj.items():
 
+        # -----------------------
         # check types (str, dict)
+
         c1 = isinstance(k0, str) and isinstance(v0, dict)
         if not c1:
             dc[k0] = "type(key) != str or type(value) != dict"
             continue
 
-        # check each key / value
-        lc2 = [
-            f'\t- {str(k1)}: type {type(v1)}'
-            f', key already in dobj0: {k1 in dobj0.get(k0, {}).keys()}'
-            for k1, v1 in v0.items()
-            if not (
-                (k1 is None or isinstance(k1, str))
-                and isinstance(v1, dict)
-                and k1 not in dobj0.get(k0, {}).keys()
-            )
-        ]
-        if len(lc2) > 0:
-            dc[k0] = (
-                f"The following keys of dobj['{k0}'] are not valid:\n"
-                + "\n".join(lc2)
-            )
-            continue
 
-        # set None to default keys if any None
-        dobj2[k0] = {}
-        for k1 in v0.keys():
+        lc2 = []
+        for k1, v1 in v0.items():
 
+            # key
             key = _generic_check._obj_key(
                 d0=dobj0.get(k0, {}),
                 short=dshort.get(k0, k0[:4]),
                 key=k1,
             )
 
+            # value
+            if not isinstance(v1, dict):
+                lc2.append(f'\t- {key}: type {type(v1)}')
+                continue
+
+            # key in dobj0
+            if key in dobj0.get(k0, {}).keys():
+
+                isidentical = _generic_utils.compare_dict(
+                    d0=dobj0[k0][key],
+                    d1=v1,
+                    dname='v1',
+                    returnas=bool,
+                    verb=False,
+                )
+
+                if not isidentical:
+                    lc2.append(f"\t- '{key}' already in dobj0['{k0}']")
+                continue
+
+            # add to dobj2
+            if dobj2.get(k0) is None:
+                dobj2[k0] = {}
             dobj2[k0][key] = dict(dobj[k0][k1])
 
+
+        if len(lc2) > 0:
+            dc[k0] = (
+                f"The following keys of dobj['{k0}'] are not valid:\n"
+                + "\n".join(lc2)
+            )
+
+    # ----------------
     # Raise Exception
+    # ----------------
+
     if len(dc) > 0:
         msg = (
             "The following keys of dobj are non-conform:\n"
@@ -1553,9 +1597,9 @@ def _set_param(
     # Set value
 
     # Check value - TBC: allow list
-    ltypes = [str, int, np.integer, float, np.floating, tuple]
+    ltypes = (int, np.integer, float, str, tuple)
     lc = [
-        isinstance(value, tuple(ltypes)),
+        isinstance(value, ltypes),
         isinstance(value, (list, tuple, np.ndarray))
         and distribute is False,
         isinstance(value, (list, tuple, np.ndarray))
@@ -1569,15 +1613,15 @@ def _set_param(
     ]
     if not (value is None or any(lc)):
         msg = (
-            """
+            f"""
             Accepted types for value include:
                 - None
-                - {}: common to all
+                - {ltypes}: common to all
                 - list, np.ndarray: key by key
                 - dict of scalar / str
 
-            The length of value must match the selected keys ({})
-            """.format(ltypes, len(key))
+            The length of value must match the selected keys ({len(key)})
+            """
         )
         raise Exception(msg)
 
@@ -1690,7 +1734,7 @@ def _ind_tofrom_key(
             and (
                 (ind.dtype == np.bool and ind.size == len(dd))
                 or (
-                    ind.dtype == np.int
+                    ind.dtype == int
                     and np.all(np.isfinite(ind))
                     and np.max(ind) <= len(dd)
                 )
@@ -1798,10 +1842,11 @@ def _select(dd=None, dd_name=None, log=None, returnas=None, **kwdargs):
     # Get raw bool indices
 
     # Get list of accessible param
-    ltypes = (int, np.int_, float, np.float_)
+    ltypes = (float, int, np.integer)
     lquant = [
         kk for kk in kwdargs.keys()
-        if any([isinstance(dd[k0][kk], ltypes) for k0 in dd.keys()])
+        if any([isinstance(dd[k0][kk], ltypes) for k0 in dd.keys()
+        ])
     ]
 
     # Prepare array of bool indices and populate
@@ -1888,100 +1933,3 @@ def _select(dd=None, dd_name=None, log=None, returnas=None, **kwdargs):
                 dtype=str,
             )
     return ind
-
-
-#############################################
-#############################################
-#       show
-#############################################
-
-
-def _get_lparam_show_append(which, key, val, lparam, for_show):
-
-    c0 = (
-        callable(val)
-        or 'class' in key
-        or 'handle' in key
-        or (which == 'axes' and key == 'bck')
-        or isinstance(val, dict)
-    )
-    if key not in lparam and ((not for_show) or (for_show and not c0)):
-        lparam.append(key)
-
-
-def _get_lparam(which=None, dd=None, for_show=None):
-
-    if for_show:
-        lparam = []
-        for k0, v0 in dd.items():
-            for k1, v1 in v0.items():
-                if isinstance(v1, dict):
-                    for k2, v2 in v1.items():
-                        k3 = f'{k1}.{k2}'
-                        _get_lparam_show_append(
-                            which, k3, v2, lparam, for_show,
-                        )
-                else:
-                    _get_lparam_show_append(
-                        which, k1, v1, lparam, for_show,
-                    )
-
-    else:
-        lparam = list(list(dd.values())[0].keys())
-
-    return lparam
-
-
-def _show_get_fields(which=None, lparam=None, dshow=None):
-
-    # show dict
-    if which not in dshow.keys():
-        lk = lparam
-
-    else:
-        lk = dshow[which]
-
-        if isinstance(lk, list):
-            lk = [
-                kk for kk in dshow[which]
-                if kk in lparam
-            ]
-        elif isinstance(lk, tuple):
-            lk = [
-                kk for kk in lparam
-                if kk not in dshow[which]
-            ]
-        else:
-            msg = f"Unreckognized dshow['{which}']"
-            raise Exception(msg)
-
-    return lk
-
-def _show_extract(dobj=None, lk=None):
-
-    lv0 = []
-    for k0 in lk:
-
-        lk0 = k0.split('.')
-        for ii in range(len(lk0)):
-            if ii == 0:
-                v0 = dobj[lk0[ii]]
-            elif v0 is not None:
-                v0 = v0[lk0[ii]]
-
-        # formatting
-        if isinstance(v0, float):
-            lv0.append(f'{v0:.2e}')
-        elif isinstance(v0, np.ndarray) and v0.size == 3:
-            if v0.dtype == np.float:
-                lv0.append(
-                    np.array2string(
-                        v0,
-                        formatter={'float': lambda x: f'{x:.3e}'},
-                    ),
-                )
-            else:
-                lv0.append(str(v0))
-        else:
-            lv0.append(str(v0))
-    return lv0
