@@ -30,11 +30,14 @@ def domain_ref(
     # -----------
     # get indices
 
-    lvectu = sorted({v0['vect'] for v0 in domain.values()})
+    lvectu = sorted({
+        v0['vect'] for v0 in domain.values()
+        if v0.get('vect') is not None
+    })
 
     for vv in lvectu:
 
-        lk0 = [k0 for k0, v0 in domain.items() if v0['vect'] == vv]
+        lk0 = [k0 for k0, v0 in domain.items() if v0.get('vect') == vv]
         for k0 in lk0:
 
             if domain[k0].get('domain') is None:
@@ -61,12 +64,14 @@ def _check(
 
     # ---------
     # prepare
+    # ---------
 
     ldata = list(coll.ddata.keys())
     lref = list(coll.dref.keys())
 
     # ------------
     # domain
+    # ------------
 
     c0 = (
         isinstance(domain, dict)
@@ -80,41 +85,61 @@ def _check(
         )
         raise Exception(msg)
 
-    # ------------
+    # --------------
     # check each key
+    # --------------
 
     dfail = {}
     domain = copy.deepcopy(domain)
     for k0, v0 in domain.items():
 
-        # check ref vector
-        kwd = {'ref': k0} if k0 in lref else {'key': k0}
-        hasref, hasvect, ref, vect = coll.get_ref_vector(**kwd)[:4]
-        if not (hasref and ref is not None):
-            dfail[k0] = "No associated ref identified!"
-            continue
-        if not (hasvect and vect is not None):
-            dfail[k0] = "No associated ref vector identified!"
-            continue
-
+        # -----------
         # v0 is dict
-        if isinstance(v0, (list, tuple)):
+
+        ltyp = (list, tuple, np.ndarray)
+        if isinstance(v0, ltyp):
+            domain[k0] = {'domain': v0}
+        elif np.isscalar(v0):
             domain[k0] = {'domain': v0}
 
         c0 = (
             isinstance(domain[k0], dict)
-            and any(ss in ['ind', 'domain'] for ss in domain[k0].keys())
-            and isinstance(domain[k0].get('domain', []), (list, tuple))
-            and isinstance(domain[k0].get('ind', np.r_[0]), np.ndarray)
+            and any([ss in ['ind', 'domain'] for ss in domain[k0].keys()])
+            and (
+                isinstance(domain[k0].get('domain'), ltyp)
+                or np.isscalar(domain[k0].get('domain', 0))
+            )
+            and isinstance(domain[k0].get('ind', np.r_[0]), (np.ndarray, int))
         )
+
         if not c0:
             dfail[k0] = "must be a dict with keys ['ind', 'domain']"
             continue
 
-        # vect
-        domain[k0]['vect'] = vect
+        # ----------------
+        # check ref vector
 
+        kwd = {'ref': k0} if k0 in lref else {'key0': k0}
+        hasref, hasvect, ref, vect = coll.get_ref_vector(**kwd)[:4]
+
+        if not (hasref and ref is not None):
+            dfail[k0] = "No associated ref identified!"
+            continue
+
+        # vect
+        domain[k0]['ref'] = ref
+
+        if domain[k0].get('domain') is not None:
+            if not (hasvect and vect is not None):
+                dfail[k0] = "No associated ref vector identified!"
+                continue
+
+            # vect
+            domain[k0]['vect'] = vect
+
+        # -------
         # domain
+
         dom = domain[k0].get('domain')
         if dom is not None:
             dom, err = _check_domain(dom)
@@ -123,17 +148,22 @@ def _check(
                 continue
             domain[k0]['domain'] = dom
 
+        # -----
         # ind
+
         ind = domain[k0].get('ind')
         if ind is not None:
-            vsize = coll.ddata[vect]['data'].size
+            if np.isscalar(ind):
+                ind = np.array([ind], dtype=int)
+
+            vsize = coll.dref[ref]['size']
             if ind.dtype == bool:
                 pass
             elif 'int' in ind.dtype.name:
                 ind2 = np.zeros((vsize,), dtype=bool)
                 ind2[ind] = True
                 domain[k0]['ind'] = ind2
-                
+
             if domain[k0]['ind'].size != vsize:
                 msg = (
                     f"Wrong size for domain['{k0}']['ind']:\n"
@@ -145,12 +175,14 @@ def _check(
 
     # -----------
     # errors
+    # -----------
 
     if len(dfail) > 0:
         lstr = [f"\t- '{k0}': {v0}" for k0, v0 in dfail.items()]
         msg = (
             "The following domain keys / values are not conform:\n"
              + "\n".join(lstr)
+             + f"\nProvided:\n{domain}"
         )
         raise Exception(msg)
 
@@ -159,21 +191,29 @@ def _check(
 
 def _check_domain(dom=None):
 
+    # 3 possibilities
     lc = [
         isinstance(dom, (list, tuple))
         and len(dom) == 2
         and all(np.isscalar(dd) for dd in dom)
         and dom[0] <= dom[1],
-        all(
+        hasattr(dom, '__iter__')
+        and all(
             isinstance(dd, (list, tuple))
             and len(dd) == 2
             and all(np.isscalar(di) for di in dd)
             and dd[0] <= dd[1]
             for dd in dom
-        )
+        ),
+        np.isscalar(dom) or np.array(dom).size == 1,
     ]
+
+    # adjust
     if lc[0]:
         dom = [dom]
+    elif lc[2]:
+        if not isinstance(dom, (float, int)):
+            dom = np.array(dom).ravel()[0]
     elif not lc[1]:
         msg = "be a list of tuples or lists of len() = 2!"
         return None, msg
@@ -192,6 +232,15 @@ def _set_ind_from_domain(
     domain=None,
 ):
 
+    # ------------
+    # scalar
+
+    if np.isscalar(domain):
+        indi = np.nanargmin(np.abs(vect - domain))
+        ind = np.zeros(vect.shape, dtype=bool)
+        ind[indi] = True
+        return ind
+
     # -----------------
     # sort intervals
 
@@ -221,4 +270,4 @@ def _set_ind_from_domain(
 
     ind = ind_in & (~ind_out)
 
-    return ind
+    return ind
\ No newline at end of file
