@@ -14,7 +14,6 @@ import scipy.interpolate as scpinterp
 
 # local
 from . import _generic_check
-from . import _class1_binning
 
 
 # ##################################################################
@@ -36,6 +35,7 @@ def interpolate(
     domain=None,
     # common ref
     ref_com=None,
+    ref_vector_strategy=None,
     # parameters
     deg=None,
     deriv=None,
@@ -45,6 +45,7 @@ def interpolate(
     return_params=None,
     returnas=None,
     store=None,
+    store_keys=None,
     inplace=None,
     # debug or unit tests
     debug=None,
@@ -72,11 +73,12 @@ def interpolate(
     # check inputs
 
     # keys
-    keys, ref_key, daxis, dunits, _ = _class1_binning._check_keys(
+    keys, ref_key, daxis, dunits, _ = _check_keys(
         coll=coll,
         keys=keys,
         ref_key=ref_key,
         only1d=False,
+        ref_vector_strategy=ref_vector_strategy,
     )
 
     # params
@@ -86,6 +88,7 @@ def interpolate(
         ddata, dout, dsh_other, sli_c, sli_x, sli_v,
         log_log, nan0, grid, ndim, xunique,
         returnas, return_params, store, inplace,
+        domain,
     ) = _check(
         coll=coll,
         # interpolation base
@@ -141,7 +144,7 @@ def interpolate(
     # adjust data and ref if xunique
 
     if xunique:
-        _xunique(dout)
+        _xunique(dout, domain=domain)
 
     # --------
     # store
@@ -151,6 +154,7 @@ def interpolate(
             coll=coll,
             dout=dout,
             inplace=inplace,
+            store_keys=store_keys,
         )
 
     # -------
@@ -188,6 +192,127 @@ def interpolate(
 # #############################################################################
 
 
+def _check_keys(
+    coll=None,
+    keys=None,
+    ref_key=None,
+    only1d=None,
+    ref_vector_strategy=None,
+):
+
+    # only1d
+    only1d = _generic_check._check_var(
+        only1d, 'only1d',
+        types=bool,
+        default=True,
+    )
+
+    maxd = 1 if only1d else 2
+
+    # ---------------
+    # keys vs ref_key
+
+    # ref_key
+    if ref_key is not None:
+
+        # basic checks
+        if isinstance(ref_key, str):
+            ref_key = (ref_key,)
+
+        lref = list(coll.dref.keys())
+        ldata = list(coll.ddata.keys())
+
+        ref_key = list(_generic_check._check_var_iter(
+            ref_key, 'ref_key',
+            types=(list, tuple),
+            types_iter=str,
+            allowed=lref + ldata,
+        ))
+
+        # check vs maxd
+        if len(ref_key) > maxd:
+            msg = (
+                f"Arg ref_key shall have no more than {maxd} elements!\n"
+                f"Provided: {ref_key}"
+            )
+            raise Exception(msg)
+
+        # check vs valid vectors
+        for ii, rr in enumerate(ref_key):
+            if rr in lref:
+                kwd = {'ref': rr}
+            else:
+                kwd = {'key0': rr}
+            hasref, hasvect, ref, ref_key[ii] = coll.get_ref_vector(
+                **kwd,
+            )[:4]
+
+            if not (hasref and hasvect):
+                msg = (
+                    f"Provided ref_key[{ii}] not a valid ref or ref vector!\n"
+                    "Provided: {rr}"
+                )
+                raise Exception(msg)
+
+        lok_keys = [
+            k0 for k0, v0 in coll.ddata.items()
+            if all([coll.ddata[rr]['ref'][0] in v0['ref'] for rr in ref_key])
+        ]
+
+        if keys is None:
+            keys = lok_keys
+    else:
+        lok_keys = list(coll.ddata.keys())
+
+    # keys
+    if isinstance(keys, str):
+        keys = [keys]
+
+    keys = _generic_check._check_var_iter(
+        keys, 'keys',
+        types=list,
+        types_iter=str,
+        allowed=lok_keys,
+    )
+
+    # ref_key
+    if ref_key is None:
+        hasref, ref, ref_key, val, dkeys = coll.get_ref_vector_common(
+            keys=keys,
+            strategy=ref_vector_strategy,
+        )
+        if ref_key is None:
+            msg = (
+                f"No matching ref vector found for:\n"
+                f"\t- keys: {keys}\n"
+                f"\t- hasref: {hasref}\n"
+                f"\t- ref: {ref}\n"
+                f"\t- ddata['{keys[0]}']['ref'] = {coll.ddata[keys[0]]['ref']} "
+            )
+            raise Exception(msg)
+        ref_key = (ref_key,)
+
+    # ------------------------
+    # daxis, dunits, units_ref
+
+    # daxis
+    daxis = {
+        k0: [
+            coll.ddata[k0]['ref'].index(coll.ddata[rr]['ref'][0])
+            for rr in ref_key
+        ]
+        for k0 in keys
+    }
+
+    # dunits
+    dunits = {k0: coll.ddata[k0]['units'] for k0 in keys}
+
+    # units_ref
+    units_ref = [coll.ddata[rr]['units'] for rr in ref_key]
+
+    return keys, ref_key, daxis, dunits, units_ref
+
+
 def _check(
     coll=None,
     # interpolation base
@@ -268,9 +393,9 @@ def _check(
     )
 
     # ---------------------
-    # get dvect from domain
+    # get dref_dom from domain
 
-    domain, dvect = _get_dvect(
+    domain, dref_dom = _get_drefdom(
         coll=coll,
         domain=domain,
         ref_key=ref_key,
@@ -283,7 +408,7 @@ def _check(
         coll=coll,
         keys=keys,
         ref_key=ref_key,
-        dvect=dvect,
+        dref_dom=dref_dom,
     )
 
     # --------
@@ -298,7 +423,7 @@ def _check(
     )
 
     if ref_com is not None and domain is not None:
-        if ref_com in [coll.ddata[k0]['ref'][0] for k0 in dvect.keys()]:
+        if ref_com in list(dref_dom.keys()):
             msg = (
                 "Arg ref_com and domain cannot be applied to the same ref!\n"
                 f"\t- ref_com: {ref_com}\n"
@@ -316,8 +441,10 @@ def _check(
         x0=x0,
         daxis=daxis,
         dunits=dunits,
+        # ref com
         dref_com=dref_com,
-        dvect=dvect,
+        # domain
+        dref_dom=dref_dom,
     )
 
     # --------------
@@ -364,6 +491,7 @@ def _check(
         ddata, dout, dsh_other, sli_c, sli_x, sli_v,
         log_log, nan0, grid, ndim, xunique,
         returnas, return_params, store, inplace,
+        domain,
     )
 
 
@@ -409,14 +537,6 @@ def _check_params(
     # ----
     # ndim
 
-    if deriv not in [None, 0] and ndim > 1:
-        msg = (
-            "Arg deriv can only be used for 1d interpolations!\n"
-            f"\t- ndim: {ndim}\n"
-            f"\t- deriv: {deriv}\n"
-        )
-        raise Exception(msg)
-
     if ndim > 2:
         msg = (
             "Interpolations of more than 2 dimensions not implemented!\n"
@@ -427,12 +547,27 @@ def _check_params(
     # -----
     # deriv
 
-    deriv = _generic_check._check_var(
-        deriv, 'deriv',
-        default=0,
-        types=int,
-        allowed=[ii for ii in range(deg + 1)],
-    )
+    if ndim == 1:
+        deriv = _generic_check._check_var(
+            deriv, 'deriv',
+            default=0,
+            types=int,
+            allowed=[ii for ii in range(deg + 1)],
+        )
+
+    elif ndim == 2:
+
+        if np.isscalar(deriv):
+            deriv = (int(deriv), int(deriv))
+
+        deriv = tuple(_generic_check._check_var_iter(
+            deriv, 'deriv',
+            default=(0, 0),
+            types=(list, tuple),
+            types_iter=int,
+            size=2,
+            allowed=[ii for ii in range(deg + 1)],
+        ))
 
     # -------
     # log_log
@@ -832,56 +967,59 @@ def _x01_grid(
     return x0, x1, refx, ix, xunique
 
 
-def _get_dvect(
+def _get_drefdom(
     coll=None,
     domain=None,
     ref_key=None,
 ):
     # ----------------
-    # domain => dvect
+    # domain => dref_dom
+
+    lr_ref_key = [coll.ddata[kk]['ref'][0] for kk in ref_key]
 
     if domain is not None:
 
         # get domain
         domain = coll.get_domain_ref(domain)
 
-        # derive dvect
-        lvectu = sorted({
-            v0['vect'] for v0 in domain.values() if v0['vect'] not in ref_key
+        # derive lrefu
+        lrefu = sorted({
+            v0['ref'] for v0 in domain.values() if v0['ref'] not in lr_ref_key
         })
 
-        dvect = {
-            k0: [k1 for k1, v1 in domain.items() if v1['vect'] == k0]
-            for k0 in lvectu
+        # derive dref_dom
+        dref_dom = {
+            rr: [k1 for k1, v1 in domain.items() if v1['ref'] == rr]
+            for rr in lrefu
         }
 
         # check unicity of vect
-        dfail = {k0: v0 for k0, v0 in dvect.items() if len(v0) > 1}
+        dfail = {k0: v0 for k0, v0 in dref_dom.items() if len(v0) > 1}
         if len(dfail) > 0:
             lstr = [f"\t- '{k0}': {v0}" for k0, v0 in dfail.items()]
             msg = (
-                "Some ref vector have been specified with multiple domains!\n"
+                "Some ref have been specified with multiple domains!\n"
                 + "\n".join(lstr)
             )
             raise Exception(msg)
 
-        # build final dvect
-        dvect = {
+        # build final dref_dom
+        dref_dom = {
             k0: domain[v0[0]]['ind']
-            for k0, v0 in dvect.items()
+            for k0, v0 in dref_dom.items()
         }
 
     else:
-        dvect = None
+        dref_dom = None
 
-    return domain, dvect
+    return domain, dref_dom
 
 
 def _get_ddata(
     coll=None,
     keys=None,
     ref_key=None,
-    dvect=None,
+    dref_dom=None,
 ):
 
     # --------
@@ -893,12 +1031,12 @@ def _get_ddata(
         data = coll.ddata[k0]['data']
 
         # apply domain
-        if dvect is not None:
-            for k1, v1 in dvect.items():
-                if coll.ddata[k1]['ref'][0] in coll.ddata[k0]['ref']:
-                    ax = coll.ddata[k0]['ref'].index(coll.ddata[k1]['ref'][0])
+        if dref_dom is not None:
+            for rr, vr in dref_dom.items():
+                if rr in coll.ddata[k0]['ref']:
+                    ax = coll.ddata[k0]['ref'].index(rr)
                     sli = tuple([
-                        v1 if ii == ax else slice(None)
+                        vr if ii == ax else slice(None)
                         for ii in range(data.ndim)
                     ])
                     data = data[sli]
@@ -918,7 +1056,7 @@ def _get_dout(
     # common refs
     dref_com=None,
     # domain
-    dvect=None,
+    dref_dom=None,
 ):
 
     # -------------
@@ -937,11 +1075,11 @@ def _get_dout(
         rd = list(coll.ddata[k0]['ref'])
 
         # apply domain
-        if dvect is not None:
-            for k1, v1 in dvect.items():
-                if coll.ddata[k1]['ref'][0] in rd:
-                    ax = rd.index(coll.ddata[k1]['ref'][0])
-                    sh[ax] = len(v1) if v1.dtype == int else v1.sum()
+        if dref_dom is not None:
+            for rr, vr in dref_dom.items():
+                if rr in rd:
+                    ax = rd.index(rr)
+                    sh[ax] = len(vr) if vr.dtype == int else vr.sum()
                     rd[ax] = None
 
         # ------------------------
@@ -951,7 +1089,7 @@ def _get_dout(
             ss for ii, ss in enumerate(sh) if ii not in daxis[k0]
         ])
 
-        # ------------------
+        # ---------------
         # x shape and ref
 
         # shx, rx
@@ -1393,6 +1531,8 @@ def _interp2d(
                 )(
                     np.log(x0[slix]),
                     np.log(x1[slix]),
+                    dx=deriv[0],
+                    dy=deriv[1],
                     grid=False,
                 )
             )
@@ -1408,6 +1548,8 @@ def _interp2d(
             )(
                 x0[slix],
                 x1[slix],
+                dx=deriv[0],
+                dy=deriv[1],
                 grid=False,
             )
 
@@ -1420,16 +1562,39 @@ def _interp2d(
 # ###############################################################
 
 
-def _xunique(dout=None):
+def _xunique(dout=None, domain=None):
     """ interpolation on a single point => eliminates a ref  """
 
+    # ----------
+    # safety check
+
+    dind = {
+        k0: [jj for jj, rr in enumerate(v0['ref']) if rr is None]
+        for k0, v0 in dout.items()
+    }
+
+    # Number of Nones expected
+    ndom = 0 if domain is None else len(domain)
+    nNone = 1 + ndom
+
+    # check
+    dwrong = {k0: v0 for k0, v0 in dind.items() if len(v0) != nNone}
+    if len(dwrong) > 0:
+        lstr = [
+            f"\t- {k0}: {dout[k0]['ref']} => {v0}" for k0, v0 in dwrong.items()
+        ]
+        msg = (
+            "Interpolate unique pt => ref should have nNone = 1 + {ndom}:\n"
+            + "\n".join(lstr)
+        )
+        raise Exception(msg)
+
+    # --------------
+    # ajusting dout
 
     for k0, v0 in dout.items():
 
-        lind = [jj for jj, rr in enumerate(v0['ref']) if rr is None]
-        assert len(lind) == 1
-        i0 = lind[0]
-
+        i0 = dind[k0][0]
         sli = [slice(None) for jj in range(v0['data'].ndim)]
         sli[i0] = 0
         dout[k0]['data'] = v0['data'][tuple(sli)]
@@ -1449,6 +1614,7 @@ def _store(
     coll=None,
     dout=None,
     inplace=None,
+    store_keys=None,
 ):
 
     # -----------
@@ -1461,19 +1627,44 @@ def _store(
         ldata = list(set(itt.chain.from_iterable([
             v0['ref'] for v0 in dout.values()
         ])))
-        coll2 = coll.extract(keys=ldata, vectors=True)
+
+        coll2 = coll.extract(
+            keys=ldata,
+            inc_vectors=True,
+            return_keys=False,
+        )
+
+    # -------------
+    # store_keys
+
+    if store_keys is None:
+        store_keys = [f"{k0}_interp" for k0 in dout.keys()]
+    if isinstance(store_keys, str):
+        store_keys = [store_keys]
+
+    lout = list(coll.ddata.keys())
+    store_keys = _generic_check._check_var_iter(
+        store_keys, 'store_keys',
+        types=list,
+        types_iter=str,
+        excluded=lout,
+    )
+
+    if len(store_keys) != len(dout):
+        msg = (
+            "Nb of store_keys != nb of keys in dout!\n"
+            f"\t- store_keys:\n{store_keys}\n "
+            f"\t- dout.keys():\n{sorted(dout.keys())}\n "
+        )
+        raise Exception(msg)
 
     # ---------
     # add data
 
-    for k0, v0 in dout.items():
+    for ii, (k0, v0) in enumerate(dout.items()):
 
-        # ---------
-        # populate
-
-        # data
         coll2.add_data(
-            key=f'{k0}_interp',
+            key=store_keys[ii],
             data=v0['data'],
             ref=v0['ref'],
             units=v0['units'],
