@@ -34,6 +34,8 @@ def get_ref_vector(
     # ressources
     ddata=None,
     dref=None,
+    # data to find vector for
+    key0=None,
     # inputs
     key=None,
     ref=None,
@@ -41,6 +43,9 @@ def get_ref_vector(
     quant=None,
     name=None,
     units=None,
+    # exclude from search
+    key_exclude=None,
+    ref_exclude=None,
     # parameters
     values=None,
     indices=None,
@@ -50,6 +55,7 @@ def get_ref_vector(
 
     # ----------------
     # check inputs
+    # ----------------
 
     # ind_strict
     ind_strict = _generic_check._check_var(
@@ -58,13 +64,21 @@ def get_ref_vector(
         default=True,
     )
 
-    # key
+    # key0
     lkok = list(ddata.keys()) + [None]
-    key = _generic_check._check_var(
-        key, 'key',
+    key0 = _generic_check._check_var(
+        key0, 'key0',
         allowed=lkok,
     )
 
+    # key
+    if key is not None:
+        lkok = list(ddata.keys()) + [None]
+        key = _generic_check._check_var(
+            key, 'key',
+            allowed=lkok,
+        )
+
     # ref
     lkok = list(dref.keys()) + [None]
     ref = _generic_check._check_var(
@@ -72,8 +86,29 @@ def get_ref_vector(
         allowed=lkok,
     )
 
-    if key is None and ref is None:
-        msg = "Please provide key or ref at least!"
+    # key_exclude
+    if key_exclude is not None:
+        if isinstance(key_exclude, str):
+            key_exclude = [key_exclude]
+        key_exclude = _generic_check._check_var_iter(
+            key_exclude, 'key_exclude',
+            types=(list, tuple),
+            types_iter=str,
+        )
+
+    # ref_exclude
+    if ref_exclude is not None:
+        if isinstance(ref_exclude, str):
+            ref_exclude = [ref_exclude]
+        ref_exclude = _generic_check._check_var_iter(
+            ref_exclude, 'ref_exclude',
+            types=(list, tuple),
+            types_iter=str,
+        )
+
+    # key0 vs ref
+    if key0 is None and ref is None:
+        msg = "Please provide key0 or ref at least!"
         raise Exception(msg)
 
     # warn
@@ -85,10 +120,11 @@ def get_ref_vector(
 
     # ------------------------
     # hasref, hasvect
+    # ------------------------
 
     hasref = None
-    if ref is not None and key is not None:
-        hasref = ref in ddata[key]['ref']
+    if ref is not None and key0 is not None:
+        hasref = ref in ddata[key0]['ref']
         if not hasref:
             ref = None
     elif ref is not None:
@@ -96,11 +132,16 @@ def get_ref_vector(
 
     if hasref is True:
         refok = (ref,)
-    elif key is not None:
-        refok = ddata[key]['ref']
+    elif key0 is not None:
+        refok = ddata[key0]['ref']
 
+    # ----------------------
     # identify possible vect
+    # ----------------------
+
     if hasref is not False:
+
+
         lp = [('dim', dim), ('quant', quant), ('name', name), ('units', units)]
         lk_vect = [
             k0 for k0, v0 in ddata.items()
@@ -111,12 +152,22 @@ def get_ref_vector(
                 or (vv is not None and v0[ss] == vv)
                 for ss, vv in lp
             ])
+            and (key_exclude is None or k0 not in key_exclude)
+            and (ref_exclude is None or v0['ref'][0] not in ref_exclude)
         ]
 
+        # if key is provided
+        if key is not None and key in lk_vect:
+            lk_vect = [key]
+
+        # particular case
+        if key0 is not None and key0 in lk_vect:
+            lk_vect = [key0]
+
         # cases
         if len(lk_vect) == 0:
             if warn is True:
-                msg = "No matching vector found!"
+                msg = f"No vector found for key0 = '{key0}', ref '{ref}'!"
                 warnings.warn(msg)
             hasvect = False
 
@@ -130,27 +181,30 @@ def get_ref_vector(
                 hasref = True
 
         else:
-            if warn is True:
-                msg = (
-                    f"Multiple possible vectors found:\n{lk_vect}\n"
-                    f"\t- key: {key}\n"
-                    f"\t- ref: {ref}\n"
-                    f"\t- hasref: {hasref}\n"
-                    f"\t- refok: {refok}\n"
-                )
-                warnings.warn(msg)
+            msg = (
+                f"Multiple possible vectors found:\n{lk_vect}\n"
+                f"\t- key0: {key0}\n"
+                f"\t- ref: {ref}\n"
+                f"\t- hasref: {hasref}\n"
+                f"\t- refok: {refok}\n"
+            )
+            warnings.warn(msg)
             hasvect = False
     else:
         hasvect = False
 
+    # -------------------------
     # set hasref if not yet set
+
     if hasvect is False:
         key_vector = None
         if hasref is None:
             hasref = False
             ref = None
 
+    # ----------------------
     # consistency check
+
     assert hasref == (ref is not None)
     assert hasvect == (key_vector is not None)
 
@@ -162,6 +216,7 @@ def get_ref_vector(
 
     # -----------------
     # values vs indices
+    # -----------------
 
     dind = _get_ref_vector_values(
         dref=dref,
@@ -175,7 +230,9 @@ def get_ref_vector(
         indices=indices,
     )
 
+    # -----
     # val
+
     if dind is None:
         if key_vector is not None:
             val = ddata[key_vector]['data']
@@ -393,7 +450,7 @@ def _get_ref_vector_find_identical(
         _, hasvecti, _, key_vecti = get_ref_vector(
             ddata=ddata,
             dref=dref,
-            key=k0,
+            key0=k0,
             ref=ref,
             dim=dim,
             quant=quant,
@@ -433,16 +490,45 @@ def get_ref_vector_common(
     # inputs
     keys=None,
     # for selecting ref vector
+    key=None,
     ref=None,
     dim=None,
     quant=None,
     name=None,
     units=None,
+    # exclude from search
+    key_exclude=None,
+    ref_exclude=None,
+    # strategy for choosing common ref vector
+    strategy=None,
+    strategy_bounds=None,
     # parameters
     values=None,
     indices=None,
     ind_strict=None,
 ):
+    """ Return a common ref vector for all data
+
+    For example if several data are based on different time vectors
+    This routine helps identify a common time vector according to strategy:
+        - 'dt_min': picks the time vector with smallest time step
+        - 'dt_max': picks the time vector with largest time step
+        - key/ref:  force-pick the time vector of choice
+
+    Additionally, stragety_bounds:
+        -
+
+    Additionally, ind_strict:
+        - False: no additional constraint
+        - True:
+
+    Additionally, indices:
+        - None
+        -
+
+    Additionally values:
+
+    """
 
     # ------------
     # check inputs
@@ -470,12 +556,18 @@ def get_ref_vector_common(
         hasrefi, hasvecti, refi, key_vecti, vali, dindi = get_ref_vector(
             ddata=ddata,
             dref=dref,
-            key=k0,
+            key0=k0,
+            # select vector
+            key=key,
             ref=ref,
             dim=dim,
             quant=quant,
             name=name,
             units=units,
+            # exclude from search
+            key_exclude=key_exclude,
+            ref_exclude=ref_exclude,
+            # parameters
             values=None,
             indices=None,
         )
@@ -486,6 +578,24 @@ def get_ref_vector_common(
             }
 
     keys = list(dkeys.keys())
+    refs = [v0['ref'] for v0 in dkeys.values() if v0['ref'] is not None]
+    keysv = [v0['key_vect'] for v0 in dkeys.values() if v0['key_vect'] is not None]
+    assert len(refs) == len(keysv)
+
+    # strategy
+    strategy = _generic_check._check_var(
+        strategy, 'strategy',
+        types=str,
+        default='dt_min',
+        allowed=['dt_min', 'dt_max'] + refs + keysv,
+    )
+
+    # strategy_bounds
+    strategy_bounds = _generic_check._check_var(
+        strategy_bounds, 'strategy_bounds',
+        default='min' if strategy in ['dt_min', 'dt_max'] else False,
+        allowed=['min', False],
+    )
 
     # ------------
     # list unique ref, key_vector
@@ -499,8 +609,11 @@ def get_ref_vector_common(
 
     else:
         hasref = True
-        lrefu = list([v0['ref'] for k0, v0 in dkeys.items()])
-        lkeyu = list([v0['key_vect'] for k0, v0 in dkeys.items()])
+
+        lkeyu = list(set(keysv))
+        lrefu = []
+        for ii, kv in enumerate(lkeyu):
+            lrefu.append(refs[keysv.index(kv)])
 
         if len(lkeyu) == 1:
             key_vector = lkeyu[0]
@@ -516,35 +629,61 @@ def get_ref_vector_common(
 
     # common vector
     val = None
-    if hasref:
+    if hasref is True:
         if key_vector is None:
 
             lv = [ddata[k0]['data'] for k0 in lkeyu]
 
-            # bounds
-            b0 = np.max([np.min(vv) for vv in lv])
-            b1 = np.min([np.max(vv) for vv in lv])
-
-            # check bounds
-            if b0 >= b1:
-                msg = "Non valid common vector values could be identified!"
-                raise Exception(msg)
+            # ---------------------------
+            # Strategy for choosing ref / key_vector
 
             # check if ready-made solution exists
             ld = [np.min(np.diff(vv)) for vv in lv]
-            imin = np.argmin(np.abs(ld))
+            if strategy == 'dt_min':
+                i0 = np.argmin(np.abs(ld))
 
-            if np.all((lv[imin] >= b0) & (lv[imin] <= b1)):
-                # the finest vector is all included in bounds
-                key_vector = lkeyu[imin]
-                val = lv[imin]
+            elif strategy == 'dt_max':
+                i0 = np.argmax(np.abs(ld))
 
             else:
-                # increments
-                val = np.linspace(b0, b1, int(np.ceil((b1-b0)/ld[imin])))
-                key_vector = None
+                i0 = [
+                    ii for ii, k0 in enumerate(lkeyu)
+                    if k0 == strategy or lrefu[ii] == strategy
+                ]
+                assert len(i0) == 1
+                i0 = i0[0]
 
+            # -----------------------------
+            # strategy regarding boundaries
+
+            if strategy_bounds == 'min':
+
+                # bounds
+                b0 = np.max([np.min(vv) for vv in lv])
+                b1 = np.min([np.max(vv) for vv in lv])
+
+                # check bounds
+                if b0 >= b1:
+                    msg = "Non valid common vector values could be identified!"
+                    raise Exception(msg)
+
+                if np.all((lv[i0] >= b0) & (lv[i0] <= b1)):
+                    # the finest vector is all included in bounds
+                    key_vector = lkeyu[i0]
+                    val = lv[i0]
+
+                else:
+                    # increments
+                    val = np.linspace(b0, b1, int(np.ceil((b1-b0)/ld[i0]) + 2))
+                    # val = lv[1]
+                    key_vector = None
+            else:
+                key_vector = lkeyu[i0]
+                val = lv[i0]
+
+            # -------------
             # indices dict
+
             for k0, v0 in dkeys.items():
                 ind, indok = _get_ref_vector_nearest(
                     ddata[v0['key_vect']]['data'],
@@ -555,9 +694,11 @@ def get_ref_vector_common(
 
             iok = np.all([v0['indok'] for v0 in dkeys.values()], axis=0)
 
-            # adjust
+            # -------------------------
+            # adjust if ind_strict
+
             if not np.all(iok):
-                if key_vector is None or ind_strict:
+                if key_vector is None and ind_strict:
                     val = val[iok]
                     # TBC
                     # for k0, v0 in dkeys.items():
@@ -566,7 +707,9 @@ def get_ref_vector_common(
                     # if key_vector is not None:
                         # key_vector = None
 
+            # ----------------------------------------------
             # try to identify identical pre-existing vector
+
             if key_vector is None:
                 key_vector = _get_ref_vector_find_identical(
                     # ressources
@@ -602,6 +745,9 @@ def get_ref_vector_common(
         dkeys=dkeys,
         key_vector=key_vector,
         val=val,
+        # exclude from search
+        key_exclude=key_exclude,
+        ref_exclude=ref_exclude,
         # values, indices
         values=values,
         indices=indices,
@@ -630,6 +776,9 @@ def _get_ref_vector_common_values(
     dkeys=None,
     key_vector=None,
     val=None,
+    # exclude from search
+    key_exclude=None,
+    ref_exclude=None,
     # values, indices
     values=None,
     indices=None,
@@ -652,12 +801,16 @@ def _get_ref_vector_common_values(
             hasrefi, hasvecti, refi, key_vecti, vali, dindi = get_ref_vector(
                 ddata=ddata,
                 dref=dref,
-                key=k0,
+                key0=k0,
                 ref=ref,
                 dim=dim,
                 quant=quant,
                 name=name,
                 units=units,
+                # exclude from search
+                key_exclude=key_exclude,
+                ref_exclude=ref_exclude,
+                # parameters
                 values=values,
                 indices=indices,
             )
@@ -767,7 +920,7 @@ def _uniformize_check(
     param = _generic_check._check_var(
         param, 'param',
         types=str,
-        allowed=['dim', 'quant', 'name', 'units'],
+        allowed=['key', 'dim', 'quant', 'name', 'units'],
         default='dim',
     )
 
@@ -775,10 +928,11 @@ def _uniformize_check(
     # list availabe values for param and associated refs
     dparref = {
         k0: [
-            v1[param] for k1, v1 in coll.ddata.items()
+            (k1 if param == 'key' else v1[param])
+            for k1, v1 in coll.ddata.items()
             if v1['monot'] == (True,)
             and v1['ref'][0] == k0
-            and (lparam is None or v1[param] in lparam)
+            and (lparam is None or (param != 'key' and v1[param] in lparam))
         ]
         for k0 in refs
     }
@@ -818,7 +972,7 @@ def _uniformize_check(
     for k0, v0 in dparam.items():
         lout = [
             k1 for k1 in v0['keys']
-            if coll.get_ref_vector(key=k1, **{param: k0})[3] is None
+            if coll.get_ref_vector(key0=k1, **{param: k0})[3] is None
         ]
         if len(lout) > 0:
             dfails[k0] = lout
@@ -846,13 +1000,13 @@ def _uniformize_check(
     return keys, refs, param, dparam, dkeypar, returnas
 
 
-
 def uniformize(
     coll=None,
     keys=None,
     refs=None,
     param=None,
     lparam=None,
+    ref_vector_strategy=None,
     returnas=None,
 ):
 
@@ -880,6 +1034,7 @@ def uniformize(
     for k0, v0 in dparam.items():
         hasref, ref, key, val, dind = coll.get_ref_vector_common(
             keys=v0['keys'],
+            strategy=ref_vector_strategy,
             **{param: k0},
         )
 
@@ -966,4 +1121,4 @@ def uniformize(
     if returnas == 'dataframe':
         pass
 
-    return stu
+    return stu
\ No newline at end of file
