@@ -87,12 +87,18 @@ class DataStock2(DataStock1):
 
         if isinstance(dtype, str):
             dtype = [dtype]
+        lok = [
+            'xdata', 'ydata',
+            'data', 'data.T',
+            'xy', 'alpha',
+            'txt', 'x', 'y', 'position',
+        ]
         dtype = _generic_check._check_var_iter(
             dtype,
             'dtype',
             types=(list, tuple),
             types_iter=str,
-            allowed=['xdata', 'ydata', 'data', 'data.T', 'xy', 'alpha', 'txt']
+            allowed=lok,
         )
         if len(dtype) != nref:
             msg = (
@@ -623,7 +629,7 @@ class DataStock2(DataStock1):
                         v0['handle'].parent(),
                     )
                 except Exception as err:
-                    error = err
+                    error = "1\n" + str(err)
             elif hasattr(v0['handle'], 'parent'):
                 try:
                     v0['handle'].manager.toolbar.__init__(
@@ -631,9 +637,12 @@ class DataStock2(DataStock1):
                         v0['handle'].parent(),
                     )
                 except Exception as err:
-                    error = True
+                    if "can't initialize an object twice" in str(err):
+                        pass
+                    else:
+                        error = "2\n" + str(err)
             else:
-                error = True
+                error = "3"
 
             if error is not False:
                 import platform
@@ -642,6 +651,7 @@ class DataStock2(DataStock1):
                 lstr0 = [f"\t- {k1}" for k1 in dir(v0['handle'])]
                 lstr1 = [f"\t- {k1}" for k1 in dir(v0['handle'].manager.toolbar)]
                 msg = (
+                    "Problem with connect()\n"
                     f"platform: {platform.platform()}\n"
                     f"python: {sys.version}\n"
                     f"backend: {plt.get_backend()}\n"
@@ -651,7 +661,7 @@ class DataStock2(DataStock1):
                     + "\n".join(lstr1)
                 )
                 if error is not True:
-                    msg += '\n' + str(err)
+                    msg += '\n' + str(error)
                 warnings.warn(msg)
 
             self._dobj['canvas'][k0]['cid'] = {
@@ -1374,53 +1384,9 @@ class DataStock2(DataStock1):
     # -------------------
 
     def on_close(self, event):
-        kcan = [
-            k0 for k0, v0 in self._dobj['canvas'].items()
-            if v0['handle'] == event.canvas
-        ]
-        if len(kcan) > 1:
-            raise Exception('Several matching canvas')
-        elif len(kcan) == 1:
-
-            if len(self._dobj['canvas']) == 1:
-                self.close_all()
-
-            else:
-                lax = [
-                    k1 for k1, v1 in self._dobj['axes'].items()
-                    if v1['canvas'] == kcan[0]
-                ]
-                lmob = [
-                    k1 for k1, v1 in self._dobj['mobile'].items()
-                    if v1['axes'] in lax
-                ]
-                for k1 in lax:
-                    del self._dobj['axes'][k1]
-                for k1 in lmob:
-                    del self._dobj['mobile'][k1]
-                del self._dobj['canvas'][kcan[0]]
-
-    def close_all(self):
-
-        # close figures
-        if 'axes' in self._dobj.keys():
-            lfig = set([
-                v0['handle'].figure for v0 in self._dobj['axes'].values()
-            ])
-            for ff in lfig:
-                plt.close(ff)
-
-        # delete obj dict
-        lk = ['interactivity', 'mobile', 'key', 'canvas', 'group', 'axes']
-        for kk in lk:
-            if kk in self._dobj.keys():
-                del self._dobj[kk]
-
-        # remove interactivity-specific param in dref
-        lp = list(set(self.get_lparam(which='ref')).intersection(
-            ['indices', 'group', 'inc']
-        ))
-        self.remove_param(which='ref', param=lp)
+        self.remove_all(excluded=['canvas']) # to avoid crash
+        print("\n---- CLOSING interactive figure ----")
+        print(f"\tleft in dax: {self.get_nbytes()[0]/1000} Ko\n")
 
 
 # #############################################################################
