@@ -24,8 +24,8 @@ class DataStock3(DataStock2):
         keyZ=None,
         keyU=None,
         ind=None,
-        vmin=None,
-        vmax=None,
+        dvminmax=None,
+        dscale=None,
         cmap=None,
         aspect=None,
         nmax=None,
@@ -38,17 +38,24 @@ class DataStock3(DataStock2):
         inverty=None,
         bck=None,
         interp=None,
+        show_commands=None,
         # figure-specific
         dax=None,
         dmargin=None,
         fs=None,
+        wintit=None,
+        tit=None,
         dcolorbar=None,
         dleg=None,
         label=None,
         connect=None,
         inplace=None,
     ):
-        """ Plot the desired 2d data array as a matrix """
+        """ Plot the desired 1d, 2, 3, or 4d data array as a matrix
+
+        other axes can be plotted using monotonous (optionally uniform) arrays
+
+        """
         return _plot_as_array.plot_as_array(
             # parameters
             coll=self,
@@ -58,8 +65,8 @@ class DataStock3(DataStock2):
             keyZ=keyZ,
             keyU=keyU,
             ind=ind,
-            vmin=vmin,
-            vmax=vmax,
+            dvminmax=dvminmax,
+            dscale=dscale,
             cmap=cmap,
             aspect=aspect,
             nmax=nmax,
@@ -72,10 +79,13 @@ class DataStock3(DataStock2):
             inverty=inverty,
             bck=bck,
             interp=interp,
+            show_commands=show_commands,
             # figure-specific
             dax=dax,
             dmargin=dmargin,
             fs=fs,
+            wintit=wintit,
+            tit=tit,
             dcolorbar=dcolorbar,
             dleg=dleg,
             label=label,
@@ -102,6 +112,7 @@ class DataStock3(DataStock2):
         rotation=None,
         inverty=None,
         bck=None,
+        show_commands=None,
         # figure-specific
         dax=None,
         dmargin=None,
@@ -131,6 +142,7 @@ class DataStock3(DataStock2):
             rotation=rotation,
             inverty=inverty,
             bck=bck,
+            show_commands=show_commands,
             # figure-specific
             dax=dax,
             dmargin=dmargin,
@@ -158,6 +170,7 @@ class DataStock3(DataStock2):
         lkeys=None,
         bstr_dict=None,
         rotation=None,
+        show_commands=None,
         # figure-specific
         dax=None,
         dmargin=None,
@@ -185,6 +198,7 @@ class DataStock3(DataStock2):
             lkeys=lkeys,
             bstr_dict=bstr_dict,
             rotation=rotation,
+            show_commands=show_commands,
             # figure-specific
             dax=dax,
             dmargin=dmargin,
@@ -247,6 +261,7 @@ class DataStock3(DataStock2):
         keyB=None,
         keyX=None,
         axis=None,
+        show_commands=None,
         # customization of scatter plot
         dlim=None,
         color_dict=None,
@@ -289,6 +304,7 @@ class DataStock3(DataStock2):
             keyB=keyB,
             keyX=keyX,
             axis=axis,
+            show_commands=show_commands,
             # customization of scatter plot
             dlim=dlim,
             color_dict=color_dict,
@@ -408,4 +424,4 @@ class DataStock3(DataStock2):
 
 __all__ = [
     sorted([k0 for k0 in locals() if k0.startswith('DataStock')])[-1]
-]
+]
\ No newline at end of file
