@@ -7,6 +7,19 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 
+__all__ = [
+    '_check_var',
+    '_check_var_iter',
+    '_check_flat1darray',
+    '_check_dict_valid_keys',
+    '_check_vectbasis',
+    '_obj_key',
+    '_check_dax',
+    '_apply_dlim',
+    '_check_cmap_vminvmax',
+]
+
+
 _LALLOWED_AXESTYPES = [
     None,
     'cross', 'hor',
@@ -67,8 +80,9 @@ def _check_var(
             var = allowed[0]
         elif var not in allowed:
             msg = (
-                f"Arg {varname} must be in {allowed}!\n"
-                f"Provided: {var}"
+                f"Arg {varname} not in allowed range!\n"
+                f"Provided: {var}\n"
+                f"Allowed: {allowed}\n"
             )
             raise Exception(_complete_extra_msg(msg, extra_msg))
 
@@ -85,8 +99,9 @@ def _check_var(
     if excluded is not None:
         if var in excluded:
             msg = (
-                f"Arg {varname} must not be in {excluded}!\n"
-                f"Provided: {var}"
+                f"Arg {varname} must not be in excluded range!\n"
+                f"Provided: {var}\n"
+                f"Excluded: {excluded}\n"
             )
             raise Exception(_complete_extra_msg(msg, extra_msg))
 
@@ -111,7 +126,7 @@ def _check_var(
 
         if err is True:
             msg = (
-                f"Arg {varname} must be {sign}\n"
+                f"Arg {varname} must be of sign {sign}\n"
                 f"Provided: {var}"
             )
             raise Exception(_complete_extra_msg(msg, extra_msg))
@@ -125,6 +140,7 @@ def _check_var_iter(
     types=None,
     types_iter=None,
     default=None,
+    size=None,
     allowed=None,
     excluded=None,
     extra_msg=None,
@@ -173,6 +189,15 @@ def _check_var_iter(
             )
             raise Exception(_complete_extra_msg(msg, extra_msg))
 
+    # check size
+    if size is not None:
+        if len(var) != size:
+            msg = (
+                f"Arg {varname} must be an iterable of len = {size}\n"
+                f"Provided: {len(var)}"
+            )
+            raise Exception(_complete_extra_msg(msg, extra_msg))
+
     # check if allowed
     if allowed is not None:
         if any([vv not in allowed for vv in var]):
@@ -337,8 +362,6 @@ def _check_dict_valid_keys(
                     var[k0] = None
                     continue
 
-            vv = var.get(k0)
-
             # routine to call
             if any([ss in v0.keys() for ss in lkarray]):
                 var[k0] = _check_flat1darray(
@@ -397,7 +420,20 @@ def _check_vectbasis(
     e2=None,
     dim=None,
     tol=None,
+    direct=None,
 ):
+    """ Check a 2d or 3d set of unit vectors
+
+    Check that:
+        - vectors are defined (if None, they can be inferred)
+
+    Normalizes vectorto be unit vectors
+    Optionally (default) check the vectors forma direct basis
+    """
+
+    # ----------------
+    # check inputs
+    # ----------------
 
     # dim
     dim = _check_var(dim, 'dim', types=int, default=3, allowed=[2, 3])
@@ -405,6 +441,13 @@ def _check_vectbasis(
     # tol
     tol = _check_var(tol, 'tol', types=float, default=1.e-14, sign='>0.')
 
+    # direct
+    direct = _check_var(direct, 'direct', types=bool, default=True)
+
+    # ----------------------
+    # check what's provided
+    # ----------------------
+
     # check is provided
     if e0 is not None:
         e0 = _check_flat1darray(e0, 'e0', size=dim, dtype=float, norm=True)
@@ -413,13 +456,23 @@ def _check_vectbasis(
     if e2 is not None:
         e2 = _check_flat1darray(e2, 'e2', size=dim, dtype=float, norm=True)
 
+    # preliminary
+    allnone = all([ee is None for ee in [e0, e1, e2][:dim]])
+    if allnone is True:
+        lstr = [f"\t- {ee}" for ee in ['e0', 'e1', 'e2'][:dim]]
+        msg = (
+            f"For a basis f dimension {dim}, provide at least one of:\n"
+            + "\n".join(lstr)
+        )
+        raise Exception(msg)
+
+    # ----------------------
+    # dim = 2
+    # ----------------------
+
     # vectors
     if dim == 2:
 
-        if e0 is None and e1 is None:
-            msg = "Please provide e0 and/or e1!"
-            raise Exception(msg)
-
         # complete if missing
         if e0 is None:
             e0 = np.r_[e1[1], -e1[0]]
@@ -432,16 +485,18 @@ def _check_vectbasis(
             raise Exception(msg)
 
         # direct
-        if np.abs(np.cross(e0, e1).tolist() - 1.) < tol:
-            msg = "Non-direct basis"
-            raise Exception(msg)
+        if direct is True:
+            if np.abs(np.cross(e0, e1).tolist() - 1.) < tol:
+                msg = "Non-direct basis"
+                raise Exception(msg)
 
         return e0, e1
 
+    # ----------------------
+    # dim = 3
+    # ----------------------
+
     else:
-        if e0 is None and e1 is None and e2 is None:
-            msg = "Please provide at least e0, e1 or e2!"
-            raise Exception(msg)
 
         # complete if 2 missing
         if e0 is None and e1 is None:
@@ -449,7 +504,7 @@ def _check_vectbasis(
         elif e0 is None and e2 is None:
             e2 = _get_vertical_unitvect(ee=e1)
         elif e1 is None and e2 is None:
-            e2 = _get_vertical_unitvect(ee=e0)
+            e2 = _get_horizontal_unitvect(ee=e0)
 
         # complete if 1 missing
         if e0 is None:
@@ -479,9 +534,10 @@ def _check_vectbasis(
             raise Exception(msg)
 
         # direct
-        if not np.allclose(np.cross(e0, e1), e2, atol=tol, rtol=1e-6):
-            msg = "Non-direct basis"
-            raise Exception(msg)
+        if direct is True:
+            if not np.allclose(np.cross(e0, e1), e2, atol=tol, rtol=1e-6):
+                msg = "Non-direct basis"
+                raise Exception(msg)
 
         return e0, e1, e2
 
@@ -531,50 +587,124 @@ def _obj_key(d0=None, short=None, key=None, ndigits=None):
 # #############################################################################
 
 
-def _check_inplace(coll=None, keys=None, inplace=None):
-    """ Check key to data and inplace """
+def _check_all_broadcastable(
+    return_full_arrays=None,
+    **kwdargs,
+):
 
-    # key
-    if isinstance(keys, str):
-        keys = [keys]
-    keys = _check_var_iter(
-        keys, 'keys',
-        default=None,
-        types=list,
-        types_iter=str,
-        allowed=list(coll.ddata.keys()),
-    )
+    # -------------------
+    # return_full_arrays
+    # -------------------
 
-    # inplace
-    inplace = _check_var(
-        inplace, 'inplace',
+    return_full_arrays = _check_var(
+        return_full_arrays, 'return_full_arrays',
         types=bool,
         default=False,
     )
 
-    # extract sub-collection of necessary
-    if inplace:
-        coll2 = coll
+    # -------------------
+    # Preliminary check
+    # -------------------
+
+    dout = {}
+    dfail = {}
+    for k0, v0 in kwdargs.items():
+        try:
+            dout[k0] = np.atleast_1d(v0)
+        except Exception:
+            dfail[k0] = f"Not convertible to np.ndarray! - {v0}"
+
+    # Raise Exception
+    if len(dfail) > 0:
+        lstr = [f"\t- {k0}: {v0}" for k0, v0 in dfail.items()]
+        msg = (
+            "The following kwdargs are non-conform:\n"
+            + "\n".join(lstr)
+        )
+        raise Exception(msg)
+
+    # -------------------
+    # check ndim
+    # -------------------
+
+    dndim = {k0: v0.ndim for k0, v0 in dout.items() if v0.shape != (1,)}
+    lndim = list(set(dndim.values()))
+
+    if len(lndim) == 0:
+        # all scalar
+        if return_full_arrays:
+            return dout, (1,)
+        else:
+            return {k0: v0[0] for k0, v0 in dout.items()}, None
+
+    elif len(lndim) == 1:
+        ndim = lndim[0]
+
     else:
-        lk0 = list(keys)
-        for key in keys:
+        lstr = [f"\t- {k0}: {v0}" for k0, v0 in dndim.items()]
+        msg = (
+            "Some keyword args have non-compatible dimensions:\n"
+            + "\n".join(lstr)
+        )
+        raise Exception(msg)
 
-            # Include all data matching any single ref
-            for rr in coll._ddata[key]['ref']:
-                for k0, v0 in coll._ddata.items():
-                    if v0['ref'] == (rr,):
-                        if k0 not in lk0:
-                            lk0.append(k0)
+    # -------------------
+    # check shapes
+    # -------------------
 
-            # include all data matching all refs
-            for k0, v0 in coll._ddata.items():
-                if v0['ref'] == coll._ddata[key]['ref']:
-                    if k0 not in lk0:
-                        lk0.append(k0)
+    dfail = {}
+    shapef = np.ones((ndim,), dtype=int)
+    for k0, v0 in dout.items():
 
-        coll2 = coll.extract(lk0)
+        if v0.shape == (1,):
+            continue
 
-    return keys, inplace, coll2
+        for ii in range(ndim):
+            if v0.shape[ii] == 1:
+                pass
+            elif shapef[ii] == 1:
+                shapef[ii] = v0.shape[ii]
+            elif v0.shape[ii] == shapef[ii]:
+                pass
+            else:
+                dfail[k0] = f"Non-compatible shape = {v0.shape} (ii = {ii})"
+                continue
+
+    shapef = tuple(shapef)
+
+    # raise Exception if needed
+    if len(dfail) > 0:
+        lstr = [f"\t- {k0}: {v0}" for k0, v0 in dfail.items()]
+        msg = (
+            "The following keywords args have non-compatible shape:\n"
+            + "\n".join(lstr)
+            + f"\nReference shape: {shapef}\n"
+        )
+        raise Exception(msg)
+
+    # -------------------
+    # reshape output
+    # -------------------
+
+    if return_full_arrays is True:
+        for k0, v0 in dout.items():
+            if v0.shape == (1,):
+                dout[k0] = np.full(shapef, v0[0])
+            elif v0.shape != shapef:
+                dout[k0] = np.broadcast_to(v0, shapef)
+
+    else:
+        for k0, v0 in dout.items():
+            if v0.shape == (1,):
+                dout[k0] = v0[0]
+
+    return dout, shapef
+
+
+# #############################################################################
+# #############################################################################
+#                   Utilities for plotting
+# #############################################################################
 
 
 def _check_dax(dax=None, main=None):
@@ -703,7 +833,7 @@ def _check_lim(lim):
     if len(dfail) > 0:
         lstr = [f"\t- lim[{ii}]: {vv}" for ii, vv in dfail.items()]
         msg = (
-            "The following non-conformities in lim have been identified:\n"*
+            "The following non-conformities in lim have been identified:\n"
             + "\n".join(lstr)
         )
         raise Exception(msg)
@@ -759,7 +889,6 @@ def _apply_lim(lim=None, data=None, logic=None):
     return ind
 
 
-
 def _apply_dlim(dlim=None, logic_intervals=None, logic=None, ddata=None):
 
     # ------------
@@ -819,6 +948,7 @@ def _apply_dlim(dlim=None, logic_intervals=None, logic=None, ddata=None):
             lstr = [f"\t- {k0}: {v0}" for k0, v0 in dfail.items()]
             msg = (
                 "The following keys have non-compatible shapes:\n"
+                + "\n".join(lstr)
             )
             raise Exception(msg)
 
@@ -880,11 +1010,6 @@ def _apply_dlim(dlim=None, logic_intervals=None, logic=None, ddata=None):
 
 def _check_cmap_vminvmax(data=None, cmap=None, vmin=None, vmax=None):
     # cmap
-    c0 = (
-        cmap is None
-        or vmin is None
-        or vmax is None
-    )
     if cmap is None or vmin is None or vmax is None:
         nanmax = np.nanmax(data)
         nanmin = np.nanmin(data)
