@@ -1,17 +1,23 @@
 
 
+import itertools as itt
+from copy import deepcopy
+
+
 import numpy as np
 import scipy.sparse as scpsp
 import astropy.units as asunits
+from functools import reduce  # forward compatibility for Python 3
+import operator
 
 
 from . import _generic_check
 
 
-# #################################################################
-# #################################################################
+# ###############################################################
+# ###############################################################
 #                   Pretty printing
-# #################################################################
+# ###############################################################
 
 
 def _pretty_print_check(
@@ -25,7 +31,10 @@ def _pretty_print_check(
     returnas=None,
 ):
 
+    # -------------
     # headers
+    # -------------
+
     c0 = (
         isinstance(headers, list)
         and all([
@@ -59,7 +68,10 @@ def _pretty_print_check(
                 headers[ii] = [headers[ii]]
     nhead = [len(h0[0]) for h0 in headers]
 
+    # -------------
     # content
+    # -------------
+
     c0 = (
         isinstance(content, list)
         and all([
@@ -86,10 +98,14 @@ def _pretty_print_check(
         )
         raise Exception(msg)
 
+    # -------------
+    # options
+    # -------------
+
     # sep
     sep = _generic_check._check_var(
         sep, 'sep',
-        default=' ',
+        default='  ',
         types=str,
     )
 
@@ -234,10 +250,10 @@ def pretty_print(
         return msg
 
 
-# #################################################################
-# #################################################################
+# ###############################################################
+# ###############################################################
 #                   Compare dict
-# #################################################################
+# ###############################################################
 
 
 def _compare_dict_verb_return(dout, returnas, verb):
@@ -266,7 +282,13 @@ def _compare_dict_verb_return(dout, returnas, verb):
         return True
 
 
-def compare_dict(d0=None, d1=None, dname=None, returnas=None, verb=None):
+def compare_dict(
+    d0=None,
+    d1=None,
+    dname=None,
+    returnas=None,
+    verb=None,
+):
 
     # ------------
     # check inputs
@@ -339,10 +361,14 @@ def compare_dict(d0=None, d1=None, dname=None, returnas=None, verb=None):
 
         # numpy arrays
         elif isinstance(d0[k0], np.ndarray):
-            if d0[k0].shape != d1[k0].shape:
-                dkeys[key] = f'!= shapes ({d0[k0].shape} vs {d1[k0].shape})'
-            elif not np.allclose(d0[k0], d1[k0], equal_nan=True):
-                dkeys[key] = "not allclose"
+            msg = _compare_arrays(
+                dname=dname,
+                k0=k0,
+                d0=d0,
+                d1=d1,
+            )
+            if msg is not None:
+                dkeys[key] = msg
 
         # sparse arrays
         elif scpsp.issparse(d0[k0]):
@@ -353,16 +379,15 @@ def compare_dict(d0=None, d1=None, dname=None, returnas=None, verb=None):
 
         # lists and tuples
         elif isinstance(d0[k0], (list, tuple)):
-            try:
-                if not d0[k0] == d1[k0]:
-                    dkeys[key] = "!= list/tuple values"
-            except Exception as err:
-                msg = (
-                    f"Don't know how to handle {key}:\n"
-                    f"\t- type: {type(d0[k0])}\n"
-                    f"\t- value: {d0[k0]}\n"
-                )
-                raise NotImplementedError(msg)
+            msg = _compare_list_tuple(
+                dname=dname,
+                k0=k0,
+                d0=d0,
+                d1=d1,
+                key=key,
+            )
+            if msg is not None:
+                dkeys[key] = msg
 
         # functions
         elif callable(d0[k0]):
@@ -401,8 +426,105 @@ def compare_dict(d0=None, d1=None, dname=None, returnas=None, verb=None):
     return _compare_dict_verb_return(dkeys, returnas, verb)
 
 
-def compare_obj(obj0=None, obj1=None, returnas=None, verb=None):
-    """ Compare thje content of 2 instances """
+def _compare_arrays(
+    dname=None,
+    k0=None,
+    d0=None,
+    d1=None,
+):
+
+    msg = None
+
+    # shape
+    if d0[k0].shape != d1[k0].shape:
+        msg = f'!= shapes ({d0[k0].shape} vs {d1[k0].shape})'
+
+    # dtype
+    elif not d0[k0].dtype == d1[k0].dtype:
+        msg = f"!= dtypes ({d0[k0].dtype} vs {d1[k0].dtype})"
+
+    # special case: array of str
+    if 'str' in d0[k0].dtype.name:
+        d0flat = d0[k0].ravel().tolist()
+        d1flat = d1[k0].ravel().tolist()
+        c0 = all([ss == d1flat[ii] for ii, ss in enumerate(d0flat)])
+    else:
+        try:
+            c0 = np.allclose(d0[k0], d1[k0], equal_nan=True)
+        except Exception as err:
+            msg = (
+                f"Failed to compare 2 arrays from '{dname}':\n"
+                f"\t- d0['{k0}'] = {d0[k0]}\n"
+                f"\t- d1['{k0}'] = {d1[k0]}\n"
+            )
+            raise Exception(msg) from err
+
+        if not c0:
+            msg = "not allclose"
+
+    return msg
+
+
+def _compare_list_tuple(
+    dname=None,
+    k0=None,
+    d0=None,
+    d1=None,
+    key=None,
+):
+
+    msg = None
+
+    # length
+    if len(d0[k0]) != len(d1[k0]):
+        msg = f'!= length ({len(d0[k0])} vs {len(d1[k0])})'
+
+    # content type
+    ltyp0 = [_simple_type(type(ss)) for ss in d0[k0]]
+    ltyp1 = [_simple_type(type(ss)) for ss in d1[k0]]
+    if any([t0 != t1 for t0, t1 in zip(ltyp0, ltyp1)]):
+        msg = f"!= content type ({ltyp0} vs {ltyp1})"
+
+    # content
+    for ii in range(len(d0[k0])):
+
+        if isinstance(d0[k0][ii], np.ndarray):
+            c0 = np.allclose(d0[k0][ii], d1[k0][ii])
+
+        else:
+
+            try:
+                c0 = d0[k0][ii] == d1[k0][ii]
+            except Exception as err:
+                msg = (
+                    f"Don't know how to handle {key}:\n"
+                    f"\t- type: {type(d0[k0])}\n"
+                    f"\t- value: {d0[k0]}\n"
+                )
+                raise NotImplementedError(msg) from err
+
+        if not c0:
+            msg = f"!= content at position {ii} ({d0[k0][ii]} vs {d1[k0][ii]})"
+            break
+
+    return msg
+
+
+def _simple_type(typ):
+    return ''.join([
+        ss for ss in typ.__name__.replace('_', '')
+        if not ss.isdigit()
+    ])
+
+
+def compare_obj(
+    obj0=None,
+    obj1=None,
+    excluded=None,
+    returnas=None,
+    verb=None,
+):
+    """ Compare the content of 2 instances """
 
     # -----------
     # Check class
@@ -414,151 +536,393 @@ def compare_obj(obj0=None, obj1=None, returnas=None, verb=None):
         raise Exception(msg)
 
     # -----------
-    # Check 
+    # Check
 
     return compare_dict(
-        d0=obj0.to_dict(),
-        d1=obj1.to_dict(),
+        d0=obj0.to_dict(excluded=excluded, returnas='values', copy=False),
+        d1=obj1.to_dict(excluded=excluded, returnas='values', copy=False),
         dname=None,
         returnas=returnas,
         verb=verb,
     )
 
 
-# #################################################################
-# #################################################################
-#               Flatten / reshape dict                 
-# #################################################################
+# ###############################################################
+# ###############################################################
+#               Flatten / reshape dict
+# ###############################################################
 
 
-def flatten_dict(
-        din=None, 
-        parent_key=None, 
-        sep=None,
-        asarray=None,
-        with_types=None,
-        type_str=None,
-    ):
-    """ Return a flattened version of the input dict """
+def to_dict(
+    coll=None,
+    flatten=None,
+    sep=None,
+    excluded=None,
+    # copy vs ref
+    asarray=None,
+    copy=None,
+    # dtypes
+    returnas=None,
+):
 
     # ------------
     # check inputs
 
-    # sep
-    sep = _generic_check._check_var(
-        sep, 'sep',
-        default='.',
-        types=str,
+    flatten = _generic_check._check_var(
+        flatten, 'flatten',
+        default=True,
+        types=bool,
     )
 
+    returnas = _generic_check._check_var(
+        returnas, 'returnas',
+        default='types',
+        types=str,
+        allowed=['types', 'values', 'both', 'blended'],
+    )
+
+    # ----------------------
+    # get flat key/type tree
+
+    dtypes, sep = flatten_dict_keys(
+        din=coll,
+        parent_key=None,
+        sep=sep,
+        excluded=excluded,
+    )
+
+    if returnas == 'types':
+        if flatten is False:
+            return reshape_dict(dtypes, sep=sep)
+        else:
+            return dtypes
+
+    # ---------------------------
+    # Get list of dict attributes
+
+    dout = dict_from_dtypes(
+        coll,
+        dtypes=dtypes,
+        flatten=flatten,
+        sep=sep,
+        asarray=asarray,
+        copy=copy,
+    )
+
+    # ---------
+    # return
+
+    if returnas == 'blended':
+        return _blend_dicts(
+            dbase=dout,
+            dextra=dtypes,
+            extra_key='__type',
+            flatten=flatten,
+        )
+    if returnas == 'both':
+        return dtypes, dout
+    else:
+        return dout
+
+
+def _flatten_dict_check(
+    din=None,
+    parent_key=None,
+    sep=None,
+    excluded=None,
+):
+    # ------------
+    # check inputs
+
+    # sep
+    if sep is not None:
+        sep = _generic_check._check_var(
+            sep, 'sep',
+            default='.',
+            types=str,
+        )
+
     # parent_key
     if parent_key is not None:
         parent_key = _generic_check._check_var(
             parent_key, 'parent_key',
-            default='.',
-            types=str,
+            default=('',),
+            types=(str, tuple),
         )
 
+    # excluded
+    if excluded is not None:
+        if isinstance(excluded, str):
+            excluded = ((excluded,),)
+
+        if not isinstance(excluded, (list, tuple)):
+            msg = "Arg excluded must be a tuple of tuples of str!"
+            raise Exception(msg)
+
+        if any([isinstance(ss, (str, list)) for ss in excluded]):
+            excluded = tuple([
+                tuple(ss) if isinstance(ss, list)
+                else (ss if isinstance(ss, tuple) else (ss,))
+                for ss in excluded
+            ])
+
+        c0 = (
+            isinstance(excluded, tuple)
+            and all([isinstance(tt, tuple) for tt in excluded])
+            and all([all([isinstance(ss, str) for ss in tt]) for tt in excluded])
+        )
+        if not c0:
+            msg = "Arg excluded must be a tuple of tuples of str!"
+            raise Exception(msg)
+
+    return parent_key, sep, excluded
+
+
+def flatten_dict_keys(
+    din=None,
+    parent_key=None,
+    sep=None,
+    excluded=None,
+):
+    """ Return a flattened version of the input dict keys"""
+
+    # ------------
+    # check inputs
+    # ------------
+
+    parent_key, sep, excluded = _flatten_dict_check(
+        din=din,
+        parent_key=parent_key,
+        sep=sep,
+        excluded=excluded,
+    )
+
+    # ------------
+    # top level
+    # ------------
+
+    if isinstance(din, dict):
+
+        dkeys = {}
+        for k0, v0 in din.items():
+
+            # key
+            if parent_key is None:
+                key = (k0,)
+            else:
+                key = tuple([k1 for k1 in parent_key] + [k0])
+
+            # value
+            if isinstance(v0, dict):
+                dkeys.update(
+                    flatten_dict_keys(
+                        v0,
+                        key,
+                        sep=None,
+                        excluded=excluded,
+                    )[0]
+                )
+
+            else:
+
+                # get class
+                if excluded is None or key not in excluded:
+                    dkeys[key] = v0.__class__.__name__
+
+    else:
+        dkeys = {}
+        lk0 = [
+            k0 for k0 in dir(din)
+            if k0 != '__dict__'
+            and '__dlinks' not in k0
+            and not (
+                hasattr(din.__class__, k0)
+                and isinstance(getattr(din.__class__, k0), property)
+            )
+            and isinstance(getattr(din, k0), dict)
+        ]
+        for k0 in lk0:
+            dkeys.update(flatten_dict_keys(
+                getattr(din, k0),
+                parent_key=(k0,),
+                sep=None,
+                excluded=excluded,
+            )[0])
+
+    # ---------------------
+    # format keys using sep
+    # ---------------------
+
+    if sep is not None:
+
+        # --------------------
+        # safety check vs sep
+
+        # dict of non-conform keys
+        dkout = {
+            k0: v0 for k0, v0 in dkeys.items()
+            if any([sep in k1 for k1 in k0])
+        }
+
+        # error msg
+        if len(dkout) > 0:
+            lstr = [f"\t- {k0}: {v0}" for k0, v0 in dkout.items()]
+            msg = (
+                f"The following keys already have the desired sep '{sep}':\n"
+                + '\n'.join(lstr)
+            )
+            raise Exception(msg)
+
+        # ----------
+        # formatting
+
+        dkeys = {sep.join(k0): v0 for k0, v0 in dkeys.items()}
+
+    return dkeys, sep
+
+
+def getFromDict(dataDict, mapList):
+    return reduce(operator.getitem, mapList, dataDict)
+
+
+def setInDict(dataDict, mapList, value):
+    getFromDict(dataDict, mapList[:-1])[mapList[-1]] = value
+
+
+def dict_from_dtypes(
+    coll=None,
+    dtypes=None,
+    flatten=None,
+    sep=None,
+    asarray=None,
+    copy=None,
+):
+    """ Assumes dtypes is flat """
+
+    # -------------------------
+    # check inputs
+
     # asarray
     asarray = _generic_check._check_var(
         asarray, 'asarray',
         default=False,
         types=bool,
     )
-    
-    # with_types
-    with_types = _generic_check._check_var(
-        with_types, 'with_types',
+
+    copy = _generic_check._check_var(
+        copy, 'copy',
         default=False,
         types=bool,
     )
 
-    # with_types
-    type_str = _generic_check._check_var(
-        type_str, 'type_str',
-        default='_type',
+    # -------------------------
+    # get flat / unflat version
+
+    # keys
+    if sep is None:
+        lkeys = sorted(dtypes.keys())
+    else:
+        lkeys = sorted([k0.split(sep) for k0 in dtypes.keys()])
+
+    # -----------
+    # build
+
+    # initialize dout
+    if flatten is True:
+        dout = {}
+    else:
+        dout = dict(reshape_dict(dtypes, sep=sep))
+
+    # loop on all keys from (flat) dtypes
+    for k0 in lkeys:
+
+        # get value from coll
+        for ii, k1 in enumerate(k0):
+            if ii == 0:
+                out = getattr(coll, k1)
+            else:
+                out = out[k1]
+
+        # asarray
+        if asarray is True:
+            out = np.asarray(out)
+
+        # set value in dout
+        if flatten is True:
+            if sep is None:
+                dout[k0] = out
+            else:
+                dout[sep.join(k0)] = out
+        else:
+            setInDict(dout, k0, out)
+
+    # ---------------
+    # prepare output
+
+    if copy is True:
+        return deepcopy(dout)
+    else:
+        return dout
+
+
+def _blend_dicts(
+    dbase=None,
+    dextra=None,
+    extra_key=None,
+    flatten=None,
+):
+
+    # -------------
+    # check inputs
+
+    extra_key = _generic_check._check_var(
+        extra_key, 'extra_key',
+        default='__type',
         types=str,
     )
 
-    # --------
-    # flatten
+    assert isinstance(dbase, dict) and isinstance(dextra, dict)
 
-    if with_types:
-        dtypes = {}
+    # ------------
+    # blend
 
-    # --------
-    # flatten
+    if flatten is True:
 
-    items = []
-    for k0, v0 in din.items():
+        for k0, v0 in dextra.items():
+            dbase[f"{k0}{extra_key}"] = v0
 
-        # key
-        if parent_key is None:
-            key = k0
-        else:
-            key = f'{parent_key}{sep}{k0}'
-
-        # value
-        if isinstance(v0, dict):
-            items.extend(
-                flatten_dict(
-                    v0,
-                    key,
-                    sep=sep,
-                    asarray=asarray,
-                    with_types=with_types,
-                    type_str=type_str,
-                ).items()
-            )
-            
-        else:
-            
-            # get class
-            cc = v0.__class__.__name__
-            
-            # store type?
-            if with_types:
-                dtypes[f'{key}{type_str}'] = cc                
-            
-            # asarray
-            if asarray:
-                
-                # astropy unit to str
-                if 'Unit' in cc:
-                    v0 = str(v0)
-
-                items.append((key, np.asarray(v0)))
-                
-            else:
-                items.append((key, v0))
-
-    # --------
-    # return
-
-    if with_types:
-        dout = dict(items)
-        dout.update(dtypes)
-        return dout
     else:
-        return dict(items)
+        msg = "Blended dict not implement for flatten = False"
+        raise NotImplementedError(msg)
+
+    return dbase
 
 
 def _reshape_dict(k0, v0, dinit={}, sep=None):
     """ Populate dinit """
 
-    lk = k0.split(sep)
-    k0 = k0 if len(lk) == 1 else lk[0]
+    assert isinstance(dinit, dict), dinit
+
+    if sep is None:
+        lk = k0
+    else:
+        lk = k0.split(sep)
+
+    k0 = lk[0]
 
     if len(lk) == 2:
         if k0 not in dinit.keys():
             dinit[k0] = {}
-        assert isinstance(dinit[k0], dict)
+        assert isinstance(dinit[k0], dict), (k0, dinit[k0])
         dinit[k0].update({lk[1]: v0})
 
     elif len(lk) > 2:
         if k0 not in dinit.keys():
             dinit[k0] = {}
-        _reshape_dict(sep.join(lk[1:]), v0, dinit=dinit[k0], sep=sep)
+        assert isinstance(dinit[k0], dict), (k0, dinit[k0])
+
+        knew = lk[1:] if sep is None else sep.join(lk[1:])
+        _reshape_dict(knew, v0, dinit=dinit[k0], sep=sep)
 
     else:
         assert k0 not in dinit.keys()
@@ -571,16 +935,59 @@ def reshape_dict(din, sep=None):
     # ------------
     # check inputs
 
-    sep = _generic_check._check_var(
-        sep, 'sep',
-        default='.',
-        types=str,
-    )
+    if sep is not None:
+        sep = _generic_check._check_var(
+            sep, 'sep',
+            default='.',
+            types=str,
+        )
 
-    # ------------
+    # ------------------------
     # Get all individual keys
 
     dout = {}
     for k0, v0 in din.items():
         _reshape_dict(k0, v0, dinit=dout, sep=sep)
+
     return dout
+
+
+# ########################################################################
+# ########################################################################
+#            Find all indices of a subsequence in sequence
+# ########################################################################
+
+
+def KnuthMorrisPratt(text, pattern):
+
+    """ Yields all starting positions of copies of the pattern in the sequence
+
+    Calling conventions are similar to string.find, but its arguments can be
+    lists or iterators, not just strings, it returns all matches, not just
+    the first one, and it does not need the whole text in memory at once.
+    Whenever it yields, it will have read the text exactly up to and including
+    the match that caused the yield.
+    """
+
+    # allow indexing into pattern and protect against change during yield
+    pattern = list(pattern)
+
+    # build table of shift amounts
+    shifts = [1] * (len(pattern) + 1)
+    shift = 1
+    for pos in range(len(pattern)):
+        while shift <= pos and pattern[pos] != pattern[pos-shift]:
+            shift += shifts[pos-shift]
+        shifts[pos+1] = shift
+
+    # do the actual search
+    startPos = 0
+    matchLen = 0
+    for c in text:
+        while matchLen == len(pattern) or \
+              matchLen >= 0 and pattern[matchLen] != c:
+            startPos += shifts[matchLen]
+            matchLen -= shifts[matchLen]
+        matchLen += 1
+        if matchLen == len(pattern):
+            yield startPos
\ No newline at end of file
