@@ -12,7 +12,8 @@ import matplotlib.colors as mcolors
 from . import _generic_check
 from . import _plot_text
 from . import _class1_compute
-from ._plot_as_array import _check_keyXYZ, _get_str_datadlab
+from ._plot_as_array import _check_keyXYZ
+from ._generic_utils_plot import _get_str_datadlab
 
 
 __all__ = ['plot_as_profile1d']
@@ -47,12 +48,15 @@ def plot_as_profile1d(
     key=None,
     key_time=None,
     keyX=None,
+    dkeys=None,
     ind=None,
+    dscale=None,
+    dvminmax=None,
     vmin=None,
     vmax=None,
-    cmap=None,
     ymin=None,
     ymax=None,
+    cmap=None,
     aspect=None,
     nmax=None,
     color_dict=None,
@@ -62,6 +66,7 @@ def plot_as_profile1d(
     rotation=None,
     inverty=None,
     bck=None,
+    show_commands=None,
     # figure-specific
     dax=None,
     dmargin=None,
@@ -70,19 +75,30 @@ def plot_as_profile1d(
     dleg=None,
     connect=None,
     inplace=None,
+    # unused
+    **kwdargs,
 ):
 
 
     # ------------
     #  check inputs
 
-    # check key, inplace flag and extract sub-collection
-    key, inplace, coll2 = _generic_check._check_inplace(
-        coll=coll,
-        keys=None if key is None else [key],
-        inplace=inplace,
+    key = _generic_check._check_var(
+        key, 'key',
+        types=str,
+        allowed=list(coll.ddata.keys()),
     )
-    key = key[0]
+
+    # check key, inplace flag and extract sub-collection
+    lk = [kk for kk in [key, key_time, keyX] if kk is not None]
+    coll2, key = coll.extract(
+        lk,
+        inc_monot=False,
+        inc_vectors=False,
+        inc_allrefs=False,
+        return_keys=True,
+    )
+    key = [kk for kk in key if kk not in [key_time, keyX]][0]
     ndim = coll2._ddata[key]['data'].ndim
 
     # --------------
@@ -173,7 +189,7 @@ def plot_as_profile1d(
         coll2.disconnect_old()
         coll2.connect()
 
-        coll2.show_commands()
+        coll2.show_commands(verb=show_commands)
         return coll2
     else:
         return coll2, dgroup
@@ -564,14 +580,16 @@ def _plot_as_profile1d(
     # ----------------------
     #  labels and data
 
-    key_time, tstr, datat, dt2, labt = _get_str_datadlab(
+    key_time, tstr, dt2, labt = _get_str_datadlab(
         keyX=key_time, nx=nt, islogX=islogtime, coll=coll,
     )
+    datat = coll.ddata[key_time]['data']
 
     # keyX can be 2d !!!
-    keyX, xstr, dataX, _, labX = _get_str_datadlab(
+    keyX, xstr, _, labX = _get_str_datadlab(
         keyX=keyX, nx=nx, islogX=None, coll=coll,
     )
+    dataX = coll.ddata[keyX]['data']
 
     # -----------------
     #  prepare slicing
@@ -953,6 +971,4 @@ def _plot_as_profile1d(
             bstr_dict=bstr_dict,
         )
 
-    return coll, dax, dgroup
-
-
+    return coll, dax, dgroup
\ No newline at end of file
