@@ -14,6 +14,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 # datastock-specific
+from .._generic_check import _check_all_broadcastable
 from .._class import DataStock
 from .._saveload import load
 
@@ -56,6 +57,8 @@ def _add_ref(st=None, nc=None, nx=None, lnt=None):
     # add references (i.e.: store size of each dimension under a unique key)
     st.add_ref(key='nc', size=nc)
     st.add_ref(key='nx', size=nx)
+    st.add_ref(key='nne', size=11)
+    st.add_ref(key='nTe', size=21)
     for ii, nt in enumerate(lnt):
         st.add_ref(key=f'nt{ii}', size=nt)
 
@@ -63,8 +66,13 @@ def _add_ref(st=None, nc=None, nx=None, lnt=None):
 def _add_data(st=None, nc=None, nx=None, lnt=None):
 
     x = np.linspace(1, 2, nx)
-    y = np.exp((x - 0.5)**2)
+    y = np.exp(-(x - 1.5)**2//0.2**2)
     y[-5] = np.nan
+
+    ne = np.logspace(15, 21, 11)
+    Te = np.logspace(1, 5, 21)
+    pec = np.exp(-(ne[:, None] - 1e18)**2/1e5**2 - (Te[None, :] - 5e3)**2/3e3**2)
+
     lt = [np.linspace(1, 10, nt) for nt in lnt]
     lprof = [(1 + np.cos(t)[:, None]) * x[None, :] for t in lt]
     lprof[0][10, -5] = np.nan
@@ -88,6 +96,34 @@ def _add_data(st=None, nc=None, nx=None, lnt=None):
         ref='nx',
     )
 
+    # ne, Te and pec
+    st.add_data(
+        key='ne',
+        data=ne,
+        dim='density',
+        quant='ne',
+        units='1/m3',
+        ref='nne',
+    )
+
+    st.add_data(
+        key='Te',
+        data=Te,
+        dim='temperature',
+        quant='Te',
+        units='eV',
+        ref='nTe',
+    )
+
+    st.add_data(
+        key='pec',
+        data=pec,
+        dim='pec',
+        quant='pec',
+        units='ph/s/sr',
+        ref=('nne', 'nTe'),
+    )
+
     for ii, nt in enumerate(lnt):
         st.add_data(
             key=f't{ii}',
@@ -139,6 +175,17 @@ def _add_data(st=None, nc=None, nx=None, lnt=None):
         ref=('nc', 'nt0', 'nx'),
     )
 
+    # add 4d array
+    st.add_data(
+        key='4d',
+        data=(
+            np.arange(nc)[:, None, None, None]
+            + lprof[0][None, :, :, None] * lt[1][None, None, None, :]
+        ),
+        dim='blabla',
+        ref=('nc', 'nt0', 'nx', 'nt1'),
+    )
+
 
 def _add_obj(st=None, nc=None):
     for ii in range(nc):
@@ -182,6 +229,46 @@ class Test01_Instanciate():
     def test03_add_obj(self):
         _add_obj(st=self.st, nc=self.nc)
 
+    # ------------------------
+    #   Tools
+    # ------------------------
+
+    def test04_check_all_broadcastable(self):
+        # all scalar
+        dout, shape = _check_all_broadcastable(a=1, b=2)
+
+        # scalar + arrays
+        dout, shape = _check_all_broadcastable(a=1, b=(1, 2, 3))
+
+        # all arrays
+        dout, shape = _check_all_broadcastable(
+            a=(1, 2, 3),
+            b=(1, 2, 3),
+        )
+
+        # all arrays - 2d
+        dout, shape = _check_all_broadcastable(
+            a=np.r_[1, 2, 3][:, None],
+            b=np.r_[10, 20][None, :],
+        )
+
+        # check flag
+        err = False
+        try:
+            dout, shape = _check_all_broadcastable(a=(1, 2), b=(1, 2, 3))
+        except Exception:
+            err = True
+        assert err is True
+
+        # all arrays - mix
+        dout, shape = _check_all_broadcastable(
+            a=np.r_[1, 2, 3][:, None],
+            b=np.r_[10, 20][None, :],
+            c=3,
+            return_full_arrays=True,
+        )
+        assert all([v0.shape == (3, 2) for v0 in dout.values()])
+
 
 #######################################################
 #
@@ -259,13 +346,27 @@ class Test02_Manipulate():
         self.st.show()
         self.st.show_data()
         self.st.show_obj()
-        self.st.show_interactive()
+
+    # ------------------------
+    #   dcolor
+    # ------------------------
+
+    def test06_get_dcolor_touch(self):
+        xx = np.arange(50)
+        aa = np.exp(-(xx[:, None]-25)**2/10**2 - (xx[None, :]-25)**2/10**2)
+        ind = (aa>0.3) & (np.arange(50)[None, :] > 25)
+        dcolor = self.st.get_color_touch(
+            aa,
+            dcolor={'foo': {'ind': ind, 'color': 'r'}}
+        )
+        assert dcolor['color'].shape == aa.shape + (4,)
+        assert dcolor['meaning'][(1.0, 0.0, 0.0)] == ['foo']
 
     # ------------------------
     #   Interpolate
     # ------------------------
 
-    def test06_get_ref_vector(self):
+    def test07_get_ref_vector(self):
         (
             hasref, hasvector,
             ref, key_vector,
@@ -280,13 +381,13 @@ class Test02_Manipulate():
         assert values.size == dind['ind'].size == 4
         assert dind['indr'].shape == (2, 4)
 
-    def test07_get_ref_vector_common(self):
+    def test08_get_ref_vector_common(self):
         hasref, ref, key, val, dout = self.st.get_ref_vector_common(
             keys=['t0', 'prof0', 'prof1', 't3'],
             dim='time',
         )
 
-    def test08_domain_ref(self):
+    def test09_domain_ref(self):
 
         domain = {
             'nx': [1.5, 2],
@@ -302,30 +403,59 @@ class Test02_Manipulate():
         lk = list(domain.keys())
         assert all([isinstance(dout[k0]['ind'], np.ndarray) for k0 in lk])
 
-    def test09_binning(self):
+    def test10_binning(self):
 
-        bins = np.linspace(1, 5, 10)
+        bins = np.linspace(1, 5, 8)
         lk = [
-            ('y', None, 0),
-            ('y', 'nx', 0),
-            (None, 'nt0', 0),
-            ('prof0', 'nt0', 0),
-            ('prof0', 'x', 1),
+            ('y', 'nx', bins, 0, False, False, 'y_bin0'),
+            ('y', 'nx', bins, 0, True, False, 'y_bin1'),
+            ('y', 'nx', 'x', 0, False, True, 'y_bin2'),
+            ('y', 'nx', 'x', 0, True, True, 'y_bin3'),
+            ('prof0', 'x', 'nt0', 1, False, True, 'p0_bin0'),
+            ('prof0', 'x', 'nt0', 1, True, True, 'p0_bin1'),
+            ('prof0-bis', 'prof0', 'x', [0, 1], False, True, 'p1_bin0'),
         ]
 
-        for (k0, kr, ax) in lk:
+        for ii, (k0, kr, kb, ax, integ, store, kbin) in enumerate(lk):
             dout = self.st.binning(
-                keys=k0,
-                ref_key=kr,
-                bins=bins,
+                data=k0,
+                bin_data0=kr,
+                bins0=kb,
+                axis=ax,
+                integrate=integ,
+                store=store,
+                store_keys=kbin,
+                safety_ratio=0.95,
+                returnas=True,
             )
 
-            k0 = list(dout.keys())[0]
-            shape = list(self.st.ddata[k0]['data'].shape)
-            shape[ax] = bins.size - 1
-            assert dout[k0]['data'].shape == tuple(shape)
+            if np.isscalar(ax):
+                ax = [ax]
 
-    def test10_interpolate(self):
+            if isinstance(kb, str):
+                if kb in self.st.ddata:
+                    nb = self.st.ddata[kb]['data'].size
+                else:
+                    nb = self.st.dref[kb]['size']
+            else:
+                nb = bins.size
+
+            k0 = list(dout.keys())[0]
+            shape = [
+                ss for ii, ss in enumerate(self.st.ddata[k0]['data'].shape)
+                if ii not in ax
+            ]
+
+            shape.insert(ax[0], nb)
+            if dout[k0]['data'].shape != tuple(shape):
+                msg = (
+                    "Mismatching shapes for case {ii}!\n"
+                    f"\t- dout['{k0}']['data'].shape = {dout[k0]['data'].shape}\n"
+                    f"\t- expected: {tuple(shape)}"
+                )
+                raise Exception(msg)
+
+    def test11_interpolate(self):
 
         lk = ['y', 'y', 'prof0', 'prof0', 'prof0', '3d']
         lref = [None, 'nx', 't0', ['nt0', 'nx'], ['t0', 'x'], ['t0', 'x']]
@@ -369,7 +499,7 @@ class Test02_Manipulate():
                 msg = str(dout[kk]['data'].shape, shape, kk, rr)
                 raise Exception(msg)
 
-    def test11_interpolate_common_refs(self):
+    def test12_interpolate_common_refs(self):
         lk = ['3d', '3d', '3d']
         lref = ['t0', ['nt0', 'nx'], ['nx']]
         lrefc = ['nc', 'nc', 'nt0']
@@ -445,17 +575,45 @@ class Test02_Manipulate():
     #   Plotting
     # ------------------------
 
-    def test12_plot_as_array(self):
+    def test13_plot_as_array_1d(self):
         dax = self.st.plot_as_array(key='t0')
+        plt.close('all')
+        del dax
+
+    def test14_plot_as_array_2d(self):
         dax = self.st.plot_as_array(key='prof0')
-        dax = self.st.plot_as_array(key='3d')
         plt.close('all')
+        del dax
 
-    def test13_plot_BvsA_as_distribution(self):
-        dax = self.st.plot_BvsA_as_distribution(keyA='prof0', keyB='prof0-bis')
+    def test15_plot_as_array_2d_log(self):
+        dax = self.st.plot_as_array(
+            key='pec', keyX='ne', keyY='Te',
+            dscale={'data': 'log'},
+        )
         plt.close('all')
+        del dax
 
-    def test14_plot_as_profile1d(self):
+    def test16_plot_as_array_3d(self):
+        dax = self.st.plot_as_array(key='3d', dvminmax={'keyX': {'min': 0}})
+        plt.close('all')
+        del dax
+
+    def test17_plot_as_array_3d_ZNonMonot(self):
+        dax = self.st.plot_as_array(key='3d', keyZ='y')
+        plt.close('all')
+        del dax
+
+    def test18_plot_as_array_4d(self):
+        dax = self.st.plot_as_array(key='4d', dscale={'keyU': 'linear'})
+        plt.close('all')
+        del dax
+
+    # def test18_plot_BvsA_as_distribution(self):
+    #     dax = self.st.plot_BvsA_as_distribution(keyA='prof0', keyB='prof0-bis')
+    #     plt.close('all')
+    #     del dax
+
+    def test20_plot_as_profile1d(self):
         dax = self.st.plot_as_profile1d(
             key='prof0',
             key_time='t0',
@@ -463,31 +621,33 @@ class Test02_Manipulate():
             bck='lines',
         )
         plt.close('all')
+        del dax
 
-    def test15_plot_as_mobile_lines(self):
+    # def test20_plot_as_mobile_lines(self):
 
-        # 3d
-        dax = self.st.plot_as_mobile_lines(
-            keyX='3d',
-            keyY='3d-bis',
-            key_time='t0',
-            key_chan='x',
-        )
+    #     # 3d
+    #     dax = self.st.plot_as_mobile_lines(
+    #         keyX='3d',
+    #         keyY='3d-bis',
+    #         key_time='t0',
+    #         key_chan='x',
+    #     )
 
-        # 2d
-        dax = self.st.plot_as_mobile_lines(
-            keyX='prof2',
-            keyY='prof2-bis',
-            key_chan='nx',
-        )
+    #     # 2d
+    #     dax = self.st.plot_as_mobile_lines(
+    #         keyX='prof2',
+    #         keyY='prof2-bis',
+    #         key_chan='nx',
+    #     )
 
-        plt.close('all')
+    #     plt.close('all')
+    #     del dax
 
     # ------------------------
     #   File handling
     # ------------------------
 
-    def test16_copy_equal(self):
+    def test22_copy_equal(self):
         st2 = self.st.copy()
         assert st2 is not self.st
 
@@ -495,10 +655,15 @@ class Test02_Manipulate():
         if msg is not True:
             raise Exception(msg)
 
-    def test17_get_nbytes(self):
+    def test23_get_nbytes(self):
         nb, dnb = self.st.get_nbytes()
 
-    def test18_saveload(self, verb=False):
+    def test24_save_pfe(self, verb=False):
+        pfe = os.path.join(_PATH_OUTPUT, 'testsave.npz')
+        self.st.save(pfe=pfe, return_pfe=False)
+        os.remove(pfe)
+
+    def test25_saveload(self, verb=False):
         pfe = self.st.save(path=_PATH_OUTPUT, verb=verb, return_pfe=True)
         st2 = load(pfe, verb=verb)
         # Just to check the loaded version works fine
@@ -506,3 +671,13 @@ class Test02_Manipulate():
         if msg is not True:
             raise Exception(msg)
         os.remove(pfe)
+
+    def test26_saveload_coll(self, verb=False):
+        pfe = self.st.save(path=_PATH_OUTPUT, verb=verb, return_pfe=True)
+        st = DataStock()
+        st2 = load(pfe, coll=st, verb=verb)
+        # Just to check the loaded version works fine
+        msg = st2.__eq__(self.st, returnas=str)
+        if msg is not True:
+            raise Exception(msg)
+        os.remove(pfe)
